#include "action_list.h"

#include <stdlib.h>
#include <string.h>
#include "image.h"
#include "color.h"

#include "data_debug_print.h"

/* ActionStats */

int ActionStats_create(ActionStats * stats, const char * name)
{
	ActionStats res = NULL;
	char * chaine = NULL;
	MovingVector vector;
	
	if(name == NULL)
		return 0;
	if(! MovingVector_create(&vector, MVC_NULL, 5))
		return 0;
	if((chaine = (char *)malloc((strlen(name) + 1) * sizeof(char))) == NULL)
		return 0;
	strcpy(chaine, name);
	if((res = (ActionStats)malloc(sizeof(struct _ActionStats))) == NULL)
	{
		free(chaine);
		return 0;
	}
	res->name = chaine;
	res->action = NULL;
	res->key = SDLK_ESCAPE;
	res->key_mod = Key_Gest_create();
	res->vector = vector;
	res->press_key = 0;
	
	*stats = res;
	return 1;
}

void ActionStats_free(ActionStats * stats)
{
	if(stats == NULL || *stats == NULL)
		return;
	if((*stats)->name != NULL)
	{
		free((*stats)->name);
		(*stats)->name = NULL;
	}
	Action_free(&((*stats)->action));
	free(*stats);
	*stats = NULL;
}

void ActionStats_set_key(ActionStats stats, SDLKey key)
{
	if(stats == NULL)
		return ;
	stats->key = key;
}

void ActionStats_set_key_mod(ActionStats stats, Key_Gest_Key key_mod)
{
	if(stats == NULL)
		return ;
	stats->key_mod.key = key_mod;
}

void ActionStats_set_vector_direction(ActionStats stats, MovingVectorCoord direction)
{
	if(stats == NULL)
		return ;
	MovingVector_define_direction(&(stats->vector), direction);
}

int ActionStats_save(FILE * flow, ActionStats stats)
{
	unsigned int size;
	unsigned char is_action;
	int version = 1;
	if(flow == NULL || stats == NULL)
		return 0;
	size = strlen(stats->name);
	is_action = stats->action != NULL;
	if(! fwrite(&version, sizeof(int), 1, flow))
		return 0;
	if(! fwrite(&size, sizeof(unsigned int), 1, flow)
	|| fwrite(stats->name, sizeof(char), size + 1, flow) != size + 1
	|| ! fwrite(&(stats->key), sizeof(SDLKey), 1, flow)
	|| ! Key_Gest_save(flow, &(stats->key_mod))
	|| ! fwrite(&is_action, sizeof(unsigned char), 1, flow))
	{
		fprintf(stderr, "Erreur sauvegarde stats\n");
		return 0;
	}
	if(! MovingVector_save(flow, &(stats->vector)))
	{
		fprintf(stderr, "Erreur sauvegarde stats vector\n");
		return 0;
	}
	if(is_action)
	{
		if(! Action_save(flow, stats->action))
		{
			fprintf(stderr, "Erreur sauvegarde stats action\n");
			return 0;
		}
	}
	return 1;
}

int ActionStats_load(FILE * flow, ActionStats * stats, Bone arma)
{
	ActionStats res = NULL;
	unsigned int size;
	unsigned char is_action = 0;
	int version = 0;
	if(flow == NULL || stats == NULL)
		return 0;
	if(! ActionStats_create(&res, ""))
		return 0;
	if(! fread(&version, sizeof(int), 1, flow))
	{
		fprintf(stderr, "Erreur chargement stats version\n");
		return 0;
	}
	if(! fread(&size, sizeof(unsigned int), 1, flow))
	{
		fprintf(stderr, "Erreur chargement stats taille nom\n");
		ActionStats_free(&res);
		return 0;
	}
	free(res->name);
	res->name = NULL;
	
	if(version != 1)
	{
		fprintf(stderr, "Erreur chargement stats version %d non reconnue\n", version);
		return 0;
	}
	if((res->name = (char *)malloc((size + 1) * sizeof(char))) == NULL)
	{
		fprintf(stderr, "Erreur chargement stats allocation nom\n");
		ActionStats_free(&res);
		return 0;
	}
	if(fread(res->name, sizeof(char), size + 1, flow) != size + 1
	|| ! fread(&(res->key), sizeof(SDLKey), 1, flow)
	|| ! Key_Gest_load(flow, &(res->key_mod))
	|| ! fread(&is_action, sizeof(unsigned char), 1, flow))
	{
		fprintf(stderr, "Erreur chargement stats data\n");
		ActionStats_free(&res);
		return 0;
	}
	if(! MovingVector_load(flow, &(res->vector)))
	{
		fprintf(stderr, "Erreur chargement stats vector\n");
		return 0;
	}
	if(is_action)
	{
		if(! Action_load(flow, &(res->action), arma))
		{
			fprintf(stderr, "Erreur chargement stats action\n");
			ActionStats_free(&res);
			return 0;
		}
	}
	*stats = res;
	return 1;
}

int ActionStats_copy(ActionStats * copy, ActionStats stats, Bone copy_arma)
{
	ActionStats res = NULL;
	if(stats == NULL)
	{
		*copy = NULL;
		return 1;
	}
	if(! ActionStats_create(&res, stats->name))
	{
		fprintf(stderr, "Erreur action_stats copie allocation\n");
		return 0;
	}
	res->key = stats->key;
	res->key_mod = stats->key_mod;
	res->vector = stats->vector;
	res->press_key = stats->press_key;
	if(! Action_copy(&(res->action), stats->action, copy_arma))
	{
		fprintf(stderr, "Erreur action_stats copie action\n");
		ActionStats_free(&res);
		return 0;
	}
	*copy = res;
	return 1;
}

/* ActionList */

static int ActionList_reset_max(ActionList list)
{
	int new_max = -1;
	ActionStats * tmp = NULL;
	
	if(list->nb_actions >= list->max_nb_actions)
		new_max = (2 * list->max_nb_actions) + 1;
	else if(list->nb_actions < list->max_nb_actions / 4)
		new_max = list->max_nb_actions / 2;
	if(new_max == -1)
		return 1;
	if((tmp = (ActionStats *)realloc(list->actions, (new_max) * sizeof(ActionStats))) == NULL && new_max != 0)
		return 0;
	list->actions = tmp;
	list->max_nb_actions = new_max;
	return 1;
}

static int ActionList_menu_define_back(Menu menu)
{
	SDL_Surface * rectangle = NULL;
	SDL_Surface * rectangle_hover = NULL;
	int success = 1;
	
	if(! SDL_GetRectangleRGB(&rectangle, 250, 35, 19, 123, 188))
		return 0;
	if(! SDL_GetRectangleRGB(&rectangle_hover, 250, 35, 23, 151, 230))
	{
		SDL_FreeSurface(rectangle);
		return 0;
	}
	success &= Menu_define_Menu_List(menu, "media/arial.ttf", 25, SDL_Color_create(255, 255, 255), rectangle, rectangle_hover);
	Menu_set_origin(menu, 0, 0);
	SDL_FreeSurface(rectangle);
	SDL_FreeSurface(rectangle_hover);
	return success;
}

int ActionList_create(ActionList * list)
{
	ActionList res = NULL;
	if((res = (ActionList)malloc(sizeof(struct _ActionList))) == NULL)
		return 0;
	res->actions = NULL;
	res->nb_actions = 0;
	res->max_nb_actions = 0;
	res->menu = NULL;
	res->show_menu = 0;
	
	if(! Menu_create(&(res->menu))
	|| ! ActionList_menu_define_back(res->menu))
	{
		ActionList_free(&res);
		return 0;
	}
	
	*list = res;
	return 1;
}

void ActionList_free(ActionList * list)
{
	if(list == NULL || *list == NULL)
		return;
	ActionList_clean(*list);
	Menu_free(&((*list)->menu));
	free(*list);
	*list = NULL;
}

void ActionList_clean(ActionList list)
{
	int i;
	if(list == NULL || list->actions == NULL)
		return;
	for(i = 0; i < list->nb_actions; ++i)
		ActionStats_free(list->actions + i);
	free(list->actions);
	list->actions = NULL;
	list->nb_actions = 0;
	list->max_nb_actions = 0;
}

int ActionList_make_menu(ActionList list)
{
	int i;
	if(list == NULL)
		return 0;
	if(! Menu_alloc_elems(list->menu, list->nb_actions))
		return 0;
	for(i = 0; i < list->nb_actions; ++i)
	{
		if(! Menu_define_elem(list->menu, i, NULL, MENU_ELEMENT_TEXT, (*(list->actions + i))->name, NULL))
			return 0;
	}
	return 1;
}

int ActionList_append(ActionList list, const char * name, int * last_id)
{
	if(list == NULL)
		return 0;
	if(! ActionList_reset_max(list))
		return 0;
	if(! ActionStats_create(list->actions + list->nb_actions, name))
		return 0;
	if(last_id != NULL)
		*last_id = list->nb_actions;
	++(list->nb_actions);
	if(! ActionList_make_menu(list))
		return 0;
	return 1;
}

int ActionList_delete(ActionList list, int id)
{
	int i;
	if(list == NULL)
		return 0;
	if(list->nb_actions <= 0)
		return 0;
	if(id < 0 || id >= list->nb_actions)
		return 0;
	ActionStats_free(list->actions + id);
	for(i = id; i < list->nb_actions - 1; ++i)
		*(list->actions + i) = *(list->actions + i + 1);
	--(list->nb_actions);
	if(! ActionList_reset_max(list))
		return 0;
	if(! ActionList_make_menu(list))
		return 0;
	return 1;
}

int ActionList_delete_bone(ActionList list, Bone bone)
{
	ActionStats stats = NULL;
	int i;
	if(list == NULL || bone == NULL)
		return 0;
	for(i = 0; i < list->nb_actions; ++i)
	{
		stats = *(list->actions + i);
		if(Action_find(stats->action, bone) == NULL)
			continue;
		if(! Action_delete(&(stats->action), bone))
			return 0;
	}
	return 1;
}

Action * ActionList_get_action_adress(ActionList list, int id)
{
	ActionStats stats = NULL;
	if((stats = ActionList_get_action_stats(list, id)) == NULL)
		return NULL;
	return &(stats->action);
}

ActionStats ActionList_get_action_stats(ActionList list, int id)
{
	if(list == NULL)
		return NULL;
	if(id < 0 || id >= list->nb_actions)
		return NULL;
	return *(list->actions + id);
}

int ActionList_get_id_from_action(ActionList list, Action action)
{
	int i;
	if(list == NULL)
		return 0;
	for(i = 0; i < list->nb_actions; ++i)
	{
		if((*(list->actions + i))->action == action)
			return i;
	}
	return -1;
}

int ActionList_get_id_from_action_stats(ActionList list, ActionStats action)
{
	int i;
	if(list == NULL)
		return 0;
	for(i = 0; i < list->nb_actions; ++i)
	{
		if(*(list->actions + i) == action)
			return i;
	}
	return -1;
}

int ActionList_get_id_from_name(ActionList list, const char * name)
{
	int i;
	if(list == NULL)
		return 0;
	for(i = 0; i < list->nb_actions; ++i)
	{
		if(strcmp((*(list->actions + i))->name, name) == 0)
			return i;
	}
	return -1;
}

int ActionList_get_id_from_keys(ActionList list, SDLKey key, Key_Gest * key_mod)
{
	int i;
	if(list == NULL)
		return 0;
	for(i = 0; i < list->nb_actions; ++i)
	{
		if((*(list->actions + i))->key == key
		&& Key_Gest_equal(&((*(list->actions + i))->key_mod), key_mod))
			return i;
	}
	return -1;
}

int ActionList_set_key(ActionList list, int id, SDLKey key)
{
	ActionStats stats = NULL;
	if((stats = ActionList_get_action_stats(list, id)) == NULL)
		return 0;
	ActionStats_set_key(stats, key);
	return 1;
}

int ActionList_set_key_mod(ActionList list, int id, Key_Gest_Key key_mod)
{
	ActionStats stats = NULL;
	if((stats = ActionList_get_action_stats(list, id)) == NULL)
		return 0;
	ActionStats_set_key_mod(stats, key_mod);
	return 1;
}

int ActionList_set_vector_direction(ActionList list, int id, MovingVectorCoord direction)
{
	ActionStats stats = NULL;
	if((stats = ActionList_get_action_stats(list, id)) == NULL)
		return 0;
	ActionStats_set_vector_direction(stats, direction);
	return 1;
}

void ActionList_show_menu(ActionList list)
{
	if(list == NULL)
		return;
	list->show_menu = ! list->show_menu;
}

int ActionList_get_event(ActionList list, SDL_Event * event, ActionStats * get_action)
{
	int exit_menu = 0;
	int select_id = -1;
	if(list == NULL)
		return 0;
	if(! list->show_menu)
		return 0;
	if(! Menu_event(list->menu, event, NULL, &exit_menu, MENU_OPTION_GET_ID, &select_id, MENU_OPTION_END))
		return 0;
	if(exit_menu)
		list->show_menu = 0;
	if(select_id != -1)
	{
		if(get_action != NULL)
			*get_action = ActionList_get_action_stats(list, select_id);
		list->show_menu = 0;
	}
	return 1;
}

void ActionList_draw(ActionList list, SDL_Surface * screen)
{
	if(list == NULL)
		return;
	if(! list->show_menu)
		return;
	Menu_draw(list->menu, screen);
}

int ActionList_save(FILE * flow, ActionList list)
{
	int i;
	if(flow == NULL || list == NULL)
		return 0;
	if(! fwrite(&(list->nb_actions), sizeof(int), 1, flow)
	|| ! fwrite(&(list->max_nb_actions), sizeof(int), 1, flow))
	{
		fprintf(stderr, "Erreur sauvegarde action_list entete\n");
		return 0;
	}
	for(i = 0; i < list->nb_actions; ++i)
	{
		if(! ActionStats_save(flow, *(list->actions + i)))
		{
			fprintf(stderr, "Erreur sauvegarde action_list stat %d\n", i);
			return 0;
		}
	}
	return 1;
}

int ActionList_load(FILE * flow, ActionList list, Bone arma)
{
	int i;
	if(flow == NULL || list == NULL)
		return 0;
	ActionList_clean(list);
	if(! fread(&(list->nb_actions), sizeof(int), 1, flow)
	|| ! fread(&(list->max_nb_actions), sizeof(int), 1, flow))
	{
		fprintf(stderr, "Erreur chargement action_list entete\n");
		return 0;
	}
	if(list->max_nb_actions != 0)
	{
		if((list->actions = (ActionStats *)calloc(list->max_nb_actions, sizeof(ActionStats))) == NULL)
		{
			fprintf(stderr, "Erreur chargement action_list allocation actions\n");
			return 0;
		}
	}
	for(i = 0; i < list->nb_actions; ++i)
	{
		if(! ActionStats_load(flow, list->actions + i, arma))
		{
			fprintf(stderr, "Erreur chargement action_list action %d\n", i);
			for(--i; i >= 0; --i)
				ActionStats_free(list->actions + i);
			free(list->actions);
			list->actions = NULL;
			return 0;
		}
		
	}
	return 1;
}

int ActionList_save_in_file(const char * path, ActionList list)
{
	FILE * ecr = NULL;
	int retour;
	if((ecr = fopen(path, "w+")) == NULL)
		return 0;
	retour = ActionList_save(ecr, list);
	fclose(ecr);
	return retour;
}

int ActionList_load_in_file(const char * path, ActionList list, Bone arma)
{
	FILE * lec = NULL;
	int retour;
	if((lec = fopen(path, "w+")) == NULL)
		return 0;
	retour = ActionList_load(lec, list, arma);
	fclose(lec);
	return retour;
}

int ActionList_add_default_actions(ActionList list)
{
	int first_id = 0;
	if(! ActionList_append(list, "Avancer", &first_id)
	|| ! ActionList_append(list, "Reculer", NULL)
	|| ! ActionList_append(list, "Sauter", NULL)
	|| ! ActionList_append(list, "S'accroupir", NULL)
	|| ! ActionList_append(list, "Garde", NULL)
	|| ! ActionList_append(list, "Frapper", NULL))
		return 0;
	if(! ActionList_set_key(list, first_id, SDLK_RIGHT)		/*Avancer*/
	|| ! ActionList_set_key(list, first_id + 1, SDLK_LEFT)		/*Reculer*/
	|| ! ActionList_set_key(list, first_id + 2, SDLK_UP)		/*Sauter*/
	|| ! ActionList_set_key(list, first_id + 3, SDLK_DOWN)		/*S'accroupir*/
	|| ! ActionList_set_key(list, first_id + 4, SDLK_DOWN)		/*Garde*/
	|| ! ActionList_set_key_mod(list, first_id + 4, KGK_LCTRL)
	|| ! ActionList_set_key(list, first_id + 5, SDLK_SPACE))	/*Frapper*/
		return 0;
	ActionList_set_vector_direction(list, first_id, MVC_RIGHT);
	ActionList_set_vector_direction(list, first_id + 1, MVC_LEFT);
	ActionList_set_vector_direction(list, first_id + 2, MVC_UP);
	ActionList_set_vector_direction(list, first_id + 3, MVC_DOWN);
	return ActionList_make_menu(list);
}

int ActionList_calculate_rotate(ActionList list)
{
	int i;
	if(list == NULL)
		return 0;
	for(i = 0; i < list->nb_actions; ++i)
	{
		if(! Action_calculate_rotate_full((*(list->actions + i))->action))
			return 0;
	}
	return 1;
}

void ActionList_reverse_angle(ActionList list)
{
	int i;
	if(list == NULL)
		return;
	for(i = 0; i < list->nb_actions; ++i)
	{
		MovingVector_reverse(&((*(list->actions + i))->vector));
		Action_reverse_angle((*(list->actions + i))->action);
	}
}

int ActionList_copy(ActionList * copy, ActionList list, Bone copy_arma)
{
	ActionList res = NULL;
	int i;
	if(list == NULL)
	{
		*copy = NULL;
		return 1;
	}
	if(! ActionList_create(&res))
	{
		fprintf(stderr, "Erreur action_list copie allocation\n");
		return 0;
	}
	res->max_nb_actions = list->max_nb_actions;
	res->actions = NULL;
	if(res->max_nb_actions != 0 && (res->actions = (ActionStats *)calloc(res->max_nb_actions, sizeof(ActionStats))) == NULL)
	{
		fprintf(stderr, "Erreur action_list copie creation liste\n");
		ActionList_free(&res);
		return 0;
	}
	res->nb_actions = list->nb_actions;
	res->show_menu = list->show_menu;
	
	for(i = 0; i < res->nb_actions; ++i)
	{
		if(! ActionStats_copy(res->actions + i, *(list->actions + i), copy_arma))
		{
			fprintf(stderr, "Erreur action_list copie %d action\n", i);
			ActionList_free(&res);
			return 0;
		}
	}
	ActionList_make_menu(res);
	
	*copy = res;
	return 1;
}