#include "armature.h"

#include "math.h"
#include "generic_meca.h"

#include <stdlib.h>
#include <assert.h>

/* Creation functions */

int Bone_create(Bone * bone)
{
	Bone res = NULL;
	BinPath id = NULL;
	ChainList prop_list = NULL;
	ShapeSet shape = NULL;
	
	if(bone == NULL)
		return 0;
	if((res = (Bone)malloc(sizeof(struct _Bone))) == NULL)
		return 0;

	if(! BinPath_create(&id)
	|| ! ShapeSet_create(&shape)
	|| ! ChainList_create_BoneProp(&prop_list))
	{
		free(res);
		BinPath_free(&id);
		ShapeSet_free(&shape);
		return 0;
	}
		
	res->start.x = res->start.y = 0;
	res->end.x = res->end.y = 0;
	res->angle = 0;
	res->start_angle = 0;
	res->angle_min = 0;
	res->deny_angle_area = MINIMUM_DENY_ANGLE_AREA;
	res->child = res->brother = NULL;
	res->id = id;
	res->prop_list = prop_list;
	res->shape = shape;
	res->shape_image = NULL;
	res->in_front_of_parent = 1;
	res->hitbox = NULL;
	res->real_end.x = res->real_end.y = 0;
	res->last_real_end.x = res->last_real_end.y = 0;
	res->hit_delay = 0;
	
	*bone = res;
	
	return 1;
}

void Bone_free(Bone * bone)
{
	if(bone == NULL || *bone == NULL)
		return;
	
	Bone_break_mirror(*bone);
	BinPath_free(&((*bone)->id));
	ShapeSet_free(&((*bone)->shape));
	ChainList_free(&((*bone)->prop_list));
	Image_free(&((*bone)->shape_image));
	HitBox_free(&((*bone)->hitbox));
	
	Bone_free(&((*bone)->child));
	Bone_free(&((*bone)->brother));
	
	free(*bone);
	*bone = NULL;
}

/* Edition functions */

static int Bone_add_start_mirror(Bone bone, int vx, int vy)
{
	Bone mirror = NULL;
	if((mirror = Bone_get_mirror(bone)) != NULL)
	{
		mirror->start.x += -vx;
		mirror->start.y += vy;
		return 1;
	}
	return 0;
}

static int Bone_add_end_mirror(Bone bone, int vx, int vy)
{
	Bone mirror = NULL;
	if((mirror = Bone_get_mirror(bone)) != NULL)
	{
		mirror->end.x += -vx;
		mirror->end.y += vy;
		return 1;
	}
	return 0;
}

static int Bone_set_start_aux(Bone bone, int x, int y, int (*mirror_add)(Bone, int, int))
{
	if(bone == NULL)
		return 0;
	if(bone->start.x == x && bone->start.y == y)
		return 0;
	
	if(mirror_add != NULL)	
		(*mirror_add)(bone, x - bone->start.x, y - bone->start.y);
		
	bone->start.x = x;
	bone->start.y = y;
	
	return 1;
}

static int Bone_set_end_aux(Bone bone, int x, int y, int (*mirror_add)(Bone, int, int))
{
	if(bone == NULL)
		return 0;
	if(bone->end.x == x && bone->end.y == y)
		return 0;
		
	if(mirror_add != NULL)	
		(*mirror_add)(bone, x - bone->end.x, y - bone->end.y);
		
	bone->end.x = x;
	bone->end.y = y;
	
	return 1;
}

int Bone_set_start(Bone bone, int x, int y)
{
	if(bone == NULL)
		return 0;
	return Bone_set_start_aux(bone, x, y, NULL);
}

int Bone_set_end(Bone bone, int x, int y)
{
	if(bone == NULL)
		return 0;
	return Bone_set_end_aux(bone, x, y, NULL);
}

int Bone_add_start(Bone bone, int vx, int vy)
{
	if(bone == NULL)
		return 0;
	return Bone_set_start(bone, bone->start.x + vx, bone->start.y + vy);
}

int Bone_add_end(Bone bone, int vx, int vy)
{
	if(bone == NULL)
		return 0;
	return Bone_set_end(bone, bone->end.x + vx, bone->end.y + vy);
}

int Bone_move_to(Bone bone, int x, int y)
{
	int vx, vy;
	if(bone == NULL)
		return 0;
	vx = x - bone->start.x;
	vy = y - bone->start.y;
	return Bone_add_start(bone, vx, vy) | Bone_add_end(bone, vx, vy);
}

int Bone_set_start_with_mirror(Bone bone, int x, int y)
{
	if(bone == NULL)
		return 0;
	return Bone_set_start_aux(bone, x, y, &Bone_add_start_mirror);
}

int Bone_set_end_with_mirror(Bone bone, int x, int y)
{
	if(bone == NULL)
		return 0;
	return Bone_set_end_aux(bone, x, y, &Bone_add_end_mirror);
}

int Bone_add_start_with_mirror(Bone bone, int vx, int vy)
{
	if(bone == NULL)
		return 0;
	return Bone_set_start_with_mirror(bone, bone->start.x + vx, bone->start.y + vy);
}

int Bone_add_end_with_mirror(Bone bone, int vx, int vy)
{
	if(bone == NULL)
		return 0;
	return Bone_set_end_with_mirror(bone, bone->end.x + vx, bone->end.y + vy);
}

void Bone_set_real_end(Bone bone, int x, int y)
{
	if(bone == NULL)
		return ;
	bone->last_real_end = bone->real_end;
	bone->real_end.x = x;
	bone->real_end.y = y;
}

void Bone_set_real_start(Bone bone, int x, int y)
{
	if(bone == NULL)
		return ;
	bone->real_start.x = x;
	bone->real_start.y = y;
}

void Bone_set_real_angle(Bone bone, double angle)
{
	if(bone == NULL)
		return ;
	bone->real_angle = angle;
}

double Bone_creation_angle(Bone bone)
{
	if(bone == NULL)
		return 0;
	return rad_to_degre(angle_segment(bone->start.x, bone->start.y, bone->end.x, bone->end.y));
}

int Bone_in_deny_angle_area(Bone bone, double angle)
{
	if(bone == NULL)
		return 0;
	return in_angle_area(bone->angle_min, bone->deny_angle_area, angle + Bone_creation_angle(bone));
}

int Bone_set_angle(Bone bone, double angle)
{
	if(bone == NULL)
		return 0;
	if(Bone_in_deny_angle_area(bone, angle))
		return 0;
	bone->angle = angle;
	
	return 1;
}

int Bone_set_start_angle(Bone bone, double angle)
{
	if(bone == NULL)
		return 0;
	Bone_set_angle(bone, angle);
	bone->start_angle = bone->angle;
	
	return 1;
}

int Bone_vector_sign(Bone bone, double start_angle, double target_angle)
{
	if(bone == NULL)
		return 0;
	if(start_angle <= target_angle)
	{
		if(in_angle_area(correct_degre_angle(start_angle + Bone_creation_angle(bone)), target_angle - start_angle, bone->angle_min + bone->deny_angle_area / 2))
			return -1;
		return 1;
	}
	else
	{
		if(in_angle_area(correct_degre_angle(target_angle + Bone_creation_angle(bone)), start_angle - target_angle, bone->angle_min + bone->deny_angle_area / 2))
			return 1;
		return -1;
	}
}

int Bone_set_angle_min(Bone bone, double angle_min)
{
	if(bone == NULL)
		return 0;
	bone->angle_min = angle_min;
	
	return 1;
}

int Bone_set_deny_angle_area(Bone bone, double deny_angle_area)
{
	if(bone == NULL)
		return 0;
	if(deny_angle_area < MINIMUM_DENY_ANGLE_AREA)
		return 0;
	bone->deny_angle_area = deny_angle_area;
	
	return 1;
}

static int Bone_generic_create(void * bone)
{
	return Bone_create((Bone *)bone);
}

int Bone_append(Bone * bone, Bone * appened)
{
	if(bone == NULL)
		return 0;
	if(! TreeChildBrother_generic_append(bone, &Bone_generic_create, get_id_in_structure(child, *bone), get_id_in_structure(brother, *bone), appened))
		return 0;
	if(! BinPath_calculate_from(*bone))
		return 0;
	return 1;
}

static void Bone_generic_free(void * bone)
{
	Bone_free((Bone *)bone);
}

static void * Bone_get_next_generic(void * target, void * current_adress)
{
	Bone * bone = (Bone *)current_adress;
	Bone res = Bone_get_next((Bone)target, *bone);
	if(res == NULL)
		return NULL;
	if(res == (*bone)->child)
		return &((*bone)->child);
	return &((*bone)->brother);
}

int Bone_delete(Bone * bone, Bone del)
{
	if(bone == NULL)
		return 0;
	if(! TreeChildBrother_generic_delete(bone, del, &Bone_generic_free, get_id_in_structure(child, *bone), get_id_in_structure(brother, *bone), &Bone_get_next_generic))
		return 0;
	if(! BinPath_calculate(*bone))
		return 0;
	return 1;
}

static void Bone_simple_get_coord_add(Bone bone, int * x, int * y, double * angle)
{
	(*x) += bone->end.x;
	(*y) += bone->end.y;
}

static void Bone_rotation_get_coord_add(Bone bone, int * x, int * y, double * angle)
{
	SDL_Rect end;
	SDL_Rect start;
	SDL_Rect origin;
	
	origin.x = origin.y = 0;
	start = get_SDL_Rect_by_rotation(bone->start, origin, degre_to_rad(*angle));
	end = get_SDL_Rect_by_rotation(bone->end, bone->start, degre_to_rad(*angle = correct_degre_angle(bone->angle + *angle)));
	(*x) += end.x + (start.x - bone->start.x);
	(*y) += end.y + (start.y - bone->start.y);
}

static int Bone_getXY_of_parent_aux(Bone bone, Bone arma, int * x, int * y, int cx, int cy, double angle, void (*add_function)(Bone, int *, int *, double *))
{
	Bone next = NULL;
	if(arma == NULL)
		return 0;
	if(arma == bone)
	{
		if(x != NULL)
			*x = cx;
		if(y != NULL)
			*y = cy;
		return 1;
	}
	if((next = Bone_get_next(bone, arma)) == NULL)
		return 0;
	if(next == arma->child)
	{
		(*add_function)(arma, &cx, &cy, &angle);
		/*cx += arma->end.x;
		cy += arma->end.y;*/
	}
	return Bone_getXY_of_parent_aux(bone, next, x, y, cx, cy, angle, add_function);

	/*Bone cur = NULL;
	if(arma == NULL)
		return 0;
	cx += arma->end.x;
	cy += arma->end.y;
	for(cur = arma->child; cur != NULL; cur = cur->brother)
	{
		if(cur == bone)
		{
			if(x != NULL)
				*x = cx;
			if(y != NULL)
				*y = cy;
			return 1;
		}
		if(Bone_getXY_of_parent_aux(bone, cur, x, y, cx, cy))
			return 1;
	}
	return 0;*/
}

int Bone_getXY_of_parent(Bone bone, Bone arma, int * x, int * y)
{
	if(arma == NULL || bone == NULL)
		return 0;
	return Bone_getXY_of_parent_aux(bone, arma, x, y, 0, 0, 0, &Bone_simple_get_coord_add);
}

int Bone_getXY_of_parent_with_angles(Bone bone, Bone arma, int * x, int * y)
{
	if(arma == NULL || bone == NULL)
		return 0;
	return Bone_getXY_of_parent_aux(bone, arma, x, y, 0, 0, 0, &Bone_rotation_get_coord_add);
}

double Bone_angle_to_bone(Bone arma, Bone target)
{
	Bone next = NULL;
	if(arma == NULL)
		return 0;
	if(target == arma)
		return 0;
	if((next = Bone_get_next(target, arma)) == NULL)
		return 0;
	if(next == arma->child)
		return correct_degre_angle(arma->angle + Bone_angle_to_bone(next, target));
	return Bone_angle_to_bone(next, target);
}

int Bone_get_parent(Bone bone, Bone arma, Bone * parent)
{
	Bone cur = NULL;
	
	if(arma == NULL || bone == NULL || parent == NULL)
		return 0;
	if(bone == arma)
		*parent = NULL;	
	for(cur = arma->child; cur != NULL; cur = cur->brother)
	{
		if(cur == bone)
		{
			*parent = arma;
			return 1;
		}
		if(Bone_get_parent(bone, cur, parent))
			return 1;
	}
	return 0;
}

Bone * Bone_adress_in_armature(Bone bone, Bone * arma)
{
	Bone next = NULL;
	if(arma == NULL)
		return NULL;
	if(*arma == bone)
		return arma;
	if((next = Bone_get_next(bone, *arma)) == NULL)
		return NULL;
	if(next == (*arma)->child)
		return Bone_adress_in_armature(bone, &((*arma)->child));
	return Bone_adress_in_armature(bone, &((*arma)->brother));
	/*for(cur = &((*arma)->child); *cur != NULL; cur = &((*cur)->brother))
	{
		if((res = Bone_adress_in_armature(bone, cur)) != NULL)
			return res;
	}
	return NULL;*/
}

int Bone_goto_front(Bone bone, Bone * arma)
{
	Bone * adress = NULL;
	Bone tmp;
	if(bone == NULL)
		return 0;
	if(bone->in_front_of_parent)
	{
		if(bone->brother == NULL)
			return 0;
	}
	else
	{
		if(bone->brother == NULL || bone->brother->in_front_of_parent)
		{
			bone->in_front_of_parent = 1;
			return 1;
		}
	}
	if((adress = Bone_adress_in_armature(bone, arma)) == NULL)
		return 0;
	tmp = bone->brother;
	bone->brother = bone->brother->brother;
	tmp->brother = bone;
	*adress = tmp;
	BinPath_calculate(*arma);
	return 1;
}

int Bone_goto_back(Bone bone, Bone * arma)
{
	Bone parent = NULL;
	Bone * prec = NULL;
	if(bone == NULL)
		return 0;
	if(! Bone_get_parent(bone, *arma, &parent))
		prec = arma;
	else
		prec = &(parent->child);
	if(*prec == NULL)
		return 0;
	if(*prec == bone)
	{
		if(bone->in_front_of_parent)
		{
			bone->in_front_of_parent = 0;
			return 1;
		}
		return 0;
	}
	for(; (*prec)->brother != bone; prec = &((*prec)->brother));
	if(bone->in_front_of_parent && ! ((*prec)->in_front_of_parent))
	{
		bone->in_front_of_parent = 0;
		return 1;
	}
	(*prec)->brother = (*prec)->brother->brother;
	bone->brother = *prec;
	*prec = bone;
	BinPath_calculate(*arma);
	return 1;
}

double correct_area(double area)
{
	area = correct_degre_angle(area);
	if(area < MINIMUM_DENY_ANGLE_AREA)
		area = MINIMUM_DENY_ANGLE_AREA;
	else if(area > 360 - MINIMUM_DENY_ANGLE_AREA)
		area = 360 - MINIMUM_DENY_ANGLE_AREA;
	return area;
}

int Bone_reparent(Bone * arma, Bone bone, Bone new_parent)
{
	Bone * bone_adress = NULL;
	int p_x, p_y;
	int pn_x, pn_y;
	if(bone == NULL || new_parent == NULL)
		return 0;
	if(Bone_include_in(new_parent, bone))
		return 0;
	if(! Bone_getXY_of_parent(bone, *arma, &p_x, &p_y)
	|| ! Bone_getXY_of_parent(new_parent, *arma, &pn_x, &pn_y)
	|| (bone_adress = Bone_adress_in_armature(bone, arma)) == NULL)
		return 0;
	pn_x += new_parent->end.x;
	pn_y += new_parent->end.y;
	Bone_set_start(bone, p_x - pn_x + bone->start.x, p_y - pn_y + bone->start.y);
	Bone_set_end(bone, p_x - pn_x + bone->end.x, p_y - pn_y + bone->end.y);
	*bone_adress = (*bone_adress)->brother; /* supression of the bon in the first parent */
	bone->brother = NULL;
	for(bone_adress = &(new_parent->child); *bone_adress != NULL; bone_adress = &((*bone_adress)->brother));
	*bone_adress = bone;
	bone->in_front_of_parent = 1;
	BinPath_calculate(*arma);
	return 1;
}

int Bone_scale_aux(Bone bone, double factor)
{
	if(bone == NULL)
		return 1;
	bone->end.x = factor * bone->end.x;
	bone->end.y = factor * bone->end.y;
	bone->start.x = factor * bone->start.x;
	bone->start.y = factor * bone->start.y;
	ShapeSet_real_scale(bone->shape, factor);
	return Bone_scale_aux(bone->brother, factor)
	&& Bone_scale_aux(bone->child, factor);
}

int Bone_scale(Bone bone, double factor)
{
	SDL_Rect start, end;
	if(bone == NULL)
		return 1;
	start = bone->start;
	end = bone->end;
	if(! Bone_scale_aux(bone, factor))
		return 0;
	bone->start = start;
	bone->end.x = factor * (end.x - start.x) + start.x;
	bone->end.y = factor * (end.y - start.y) + start.y;
	return 1;
}

static int Bone_get_dim_aux(Bone bone, int * x_min, int * y_min, int * x_max, int * y_max, int x, int y)
{
	int bx_min, by_min, bx_max, by_max;
	if(bone == NULL)
		return 1;
	bx_min = min_int(2, bone->start.x, bone->end.x) + x;
	by_min = min_int(2, bone->start.y, bone->end.y) + y;
	bx_max = max_int(2, bone->start.x, bone->end.x) + x;
	by_max = max_int(2, bone->start.y, bone->end.y) + y;
	if(bx_min < *x_min)
		*x_min = bx_min;
	if(by_min < *y_min)
		*y_min = by_min;
	if(bx_max > *x_max)
		*x_max = bx_max;
	if(by_max > *y_max)
		*y_max = by_max;
	return Bone_get_dim_aux(bone->child, x_min, y_min, x_max, y_max, x + bone->end.x, y + bone->end.y)
	| Bone_get_dim_aux(bone->brother, x_min, y_min, x_max, y_max, x, y);
}

int Bone_get_dim(Bone bone, int * x_min, int * y_min, int * x_max, int * y_max)
{
	int dx_min, dy_min, dx_max, dy_max;
	if(bone == NULL)
		return 0;
	dx_min = min_int(2, bone->start.x, bone->end.x);
	dy_min = min_int(2, bone->start.y, bone->end.y);
	dx_max = max_int(2, bone->start.x, bone->end.x);
	dy_max = max_int(2, bone->start.y, bone->end.y);
	if(! Bone_get_dim_aux(bone, &dx_min, &dy_min, &dx_max, &dy_max, 0, 0))
		return 0;
	if(x_min != NULL)
		*x_min = dx_min;
	if(y_min != NULL)
		*y_min = dy_min;
	if(x_max != NULL)
		*x_max = dx_max;
	if(y_max != NULL)
		*y_max = dy_max;
	return 1;
}

static void Bone_reverse_aux(Bone bone)
{
	if(bone == NULL)
		return;
	bone->start.x *= -1;
	bone->end.x *= -1;
	bone->angle_min = correct_degre_angle(-(bone->angle_min + 90) - 90 - bone->deny_angle_area + 1);
	bone->start_angle = correct_degre_angle(-bone->start_angle);
	bone->angle = correct_degre_angle(-bone->angle);
	ShapeSet_reverse(bone->shape);
	Bone_reverse_aux(bone->brother);
	Bone_reverse_aux(bone->child);
}

void Bone_reverse(Bone bone)
{
	SDL_Rect start;
	if(bone == NULL)
		return;
	start = bone->start;
	Bone_add_start(bone, -start.x, -start.y);
	Bone_add_end(bone, -start.x, -start.y);
	Bone_reverse_aux(bone);
	Bone_add_start(bone, start.x, start.y);
	Bone_add_end(bone, start.x, start.y);
}

static int Bone_copy_aux(Bone * copy, Bone bone, Bone * start)
{
	Bone res = NULL;
	/*Bone mir = NULL;*/
	
	if(bone == NULL)
	{
		*copy = NULL;
		return 1;
	}
	if(! Bone_create(&res))
	{
		fprintf(stderr, "Erreur bone copie allocation\n");
		return 0;
	}
	BinPath_free(&(res->id));
	ShapeSet_free(&(res->shape));
	Image_free(&(res->shape_image));
	HitBox_free(&(res->hitbox));
	
	if(! BinPath_copy(&(res->id), bone->id))
	{
		fprintf(stderr, "Erreur bone copie id\n");
		Bone_free(&res);
		return 0;
	}
	if(! ShapeSet_copy(&(res->shape), bone->shape))
	{
		fprintf(stderr, "Erreur bone copie shape\n");
		Bone_free(&res);
		return 0;
	}
	if(! Image_copy(&(res->shape_image), bone->shape_image))
	{
		fprintf(stderr, "Erreur bone copie image\n");
		Bone_free(&res);
		return 0;
	}
	if(! HitBox_copy(&(res->hitbox), bone->hitbox))
	{
		fprintf(stderr, "Erreur bone copie hitbox\n");
		Bone_free(&res);
		return 0;
	}
	res->start = bone->start;
	res->end = bone->end;
	res->angle = bone->angle;
	res->start_angle = bone->start_angle;
	res->angle_min = bone->angle_min;
	res->deny_angle_area = bone->deny_angle_area;
	res->in_front_of_parent = bone->in_front_of_parent;
	
	if(! Bone_copy_aux(&(res->child), bone->child, start)
	|| ! Bone_copy_aux(&(res->brother), bone->brother, start))
	{
		Bone_free(&res);
		return 0;
	}
	/*if((mir = Bone_get_mirror(bone)) != NULL)
	{
		if(! BinPath_calculate(*start))
		{
			fprintf(stderr, "Erreur bone copie, calcul des chemins binaires\n");
			Bone_free(&res);
			return 0;
		}
		if((mir = BinPath_get_bone(mir->id, *start)) == NULL)
		{
			fprintf(stderr, "Erreur bone copie, recuperation miroir\n");
			Bone_free(&res);
			return 0;
		}
		if(! Bone_make_mirror(res, mir))
		{
			fprintf(stderr, "Erreur bone copie creation miroir\n");
			Bone_free(&res);
			return 0;
		}
	}*/
	
	*copy = res;
	return 1;
}

int Bone_copy(Bone * copy, Bone bone)
{
	Bone res = NULL;
	if(! Bone_copy_aux(&res, bone, &res))
		return 0;
	*copy = res;
	return 1;
}

static int Armature_save_aux(FILE * flow, const Bone arma)
{
	unsigned char next = 0;
	unsigned char type = 'r';
	unsigned char version = 1;
	Bone mirror = NULL;
	if(arma == NULL)
		return 1;
	mirror = Bone_get_mirror(arma);
	next = (arma->brother != NULL) | ((arma->child != NULL) << 1) | ((mirror != NULL) << 2);
	
	if(! fwrite(&type, sizeof(unsigned char), 1, flow)
	|| ! fwrite(&version, sizeof(unsigned char), 1, flow)
	|| ! fwrite(&(arma->start_angle), sizeof(double), 1, flow)
	|| ! fwrite(&next, sizeof(unsigned char), 1, flow)
	|| ! fwrite(&(arma->start), sizeof(SDL_Rect), 1, flow)
	|| ! fwrite(&(arma->end), sizeof(SDL_Rect), 1, flow)
	|| ! fwrite(&(arma->angle), sizeof(double), 1, flow)
	|| ! fwrite(&(arma->angle_min), sizeof(double), 1, flow)
	|| ! fwrite(&(arma->deny_angle_area), sizeof(double), 1, flow)
	|| ! fwrite(&(arma->in_front_of_parent), sizeof(int), 1, flow))
	{
		fprintf(stderr, "Erreur sauvegarde bone data\n");
		return 0;
	}
	if(! ShapeSet_save(flow, arma->shape))
	{
		fprintf(stderr, "Erreur sauvegarde bone shape\n");
		return 0;
	}
	if(mirror != NULL)
		BinPath_save(flow, mirror->id);
	
	if(! Armature_save_aux(flow, arma->brother))
	{
		fprintf(stderr, "Erreur sauvegarde bone brother\n");
		return 0;
	}
	if(! Armature_save_aux(flow, arma->child))
	{
		fprintf(stderr, "Erreur sauvegarde bone child\n");
		return 0;
	}
	return 1;
}

int Armature_save(FILE * flow, const Bone arma)
{
	if(flow == NULL || arma == NULL)
		return 0;
	return Armature_save_aux(flow, arma);
}

static int Armature_load_aux(FILE * flow, Bone * arma, Bone * start)
{
	unsigned char next = 0;
	unsigned char type;
	unsigned char version = 0;
	BinPath mirror_id = NULL;
	if(! Bone_create(arma))
	{
		fprintf(stderr, "Erreur chargement bone creation\n");
		return 0;
	}
	if(! fread(&type, sizeof(unsigned char), 1, flow))
	{
		fprintf(stderr, "Erreur chargement bone type symbol\n");
		goto free_zone;
	}
	
	/*
		'b' is the old version
	*/
	if(type != 'b' && type != 'r')
	{
		fprintf(stderr, "Erreur chargement bone entete id, trouvé '%c' attendu 'b' (ancienne version) ou 'r' (nouvelle version)\n", type);
		goto free_zone;
	}
	if(type == 'r')
	{
		if(! fread(&version, sizeof(unsigned char), 1, flow))
		{
			fprintf(stderr, "Erreur chargement bone(%c) version\n", type);
			goto free_zone;
		}
		if(version != 1)
		{
			fprintf(stderr, "Erreur chargement bone(%c) version %d non reconnue\n", type, version);
			goto free_zone;
		}
		if(! fread(&((*arma)->start_angle), sizeof(double), 1, flow))
		{
			fprintf(stderr, "Erreur chargement bone(%c) angle de départ\n", type);
			goto free_zone;
		}
	}
	if(! fread(&next, sizeof(unsigned char), 1, flow)
	|| ! fread(&((*arma)->start), sizeof(SDL_Rect), 1, flow)
	|| ! fread(&((*arma)->end), sizeof(SDL_Rect), 1, flow)
	|| ! fread(&((*arma)->angle), sizeof(double), 1, flow)
	|| ! fread(&((*arma)->angle_min), sizeof(double), 1, flow)
	|| ! fread(&((*arma)->deny_angle_area), sizeof(double), 1, flow)
	|| ! fread(&((*arma)->in_front_of_parent), sizeof(int), 1, flow))
	{
		fprintf(stderr, "Erreur chargement bone(%c) data\n", type);
		goto free_zone;
	}
		
	if(! ShapeSet_load_stats(flow, (*arma)->shape))
	{
		fprintf(stderr, "Erreur chargement bone(%c) shape\n", type);
		goto free_zone;
	}
	if(next & 4)
	{
		if(! BinPath_load(flow, &mirror_id))
		{
			fprintf(stderr, "Erreur chargement bone(%c) chemin binaire du miroir\n", type);
			goto free_zone;
		}
		#ifdef DEBUG
		BinPath_aff_debug(mirror_id);
		#endif
	}
	if(next & 1)
	{
		if(! Armature_load_aux(flow, &((*arma)->brother), start))
		{
			fprintf(stderr, "Erreur chargement bone(%c) frère\n", type);
			goto free_zone;
		}
	}
	if(next & 2)
	{
		if(! Armature_load_aux(flow, &((*arma)->child), start))
		{
			fprintf(stderr, "Erreur chargement bone(%c) enfant\n", type);
			goto free_zone;
		}
	}
	if(mirror_id != NULL && Bone_get_mirror(*arma) == NULL)
	{
		Bone mirror = NULL;
		if(! BinPath_calculate(*start))
		{
			fprintf(stderr, "Erreur chargement bone(%c) echec du calcul des chemins binaires\n", type);
			goto free_zone;
		}
		if((mirror = BinPath_get_bone(mirror_id, *start)) != NULL)
		{
			if(! Bone_make_mirror(*arma, mirror))
			{
				fprintf(stderr, "Erreur chargement bone(%c) création miroir\n", type);
				goto free_zone;
			}
		}
	}
	
	BinPath_free(&mirror_id);
	return 1;
	
	free_zone:;
	
	Bone_free(arma);
	BinPath_free(&mirror_id);
	return 0;
}

int Armature_load(FILE * flow, Bone * arma)
{
	Bone res = NULL;
	if(flow == NULL || arma == NULL)
		return 0;
	
	if(! Armature_load_aux(flow, &res, &res))
		return 0;
	if(! BinPath_calculate(res))
	{
		Bone_free(&res);
		return 0;
	}
	Bone_free(arma);
	*arma = res;
	return 1;
}

int Armature_save_in_file(const char * path, const Bone arma)
{
	FILE * ecr = NULL;
	int retour;
	if((ecr = fopen(path, "w+")) == NULL)
		return 0;
	retour = Armature_save(ecr, arma);
	fclose(ecr);
	return retour;
}

int Armature_load_from_file(const char * path, Bone * arma)
{
	FILE * lec = NULL;
	int retour;
	if((lec = fopen(path, "r")) == NULL)
		return 0;
	retour = Armature_load(lec, arma);
	fclose(lec);
	return retour;
}

/* BinPath */

int Bone_include_in(Bone bone, Bone arma)
{
	int val;
	if(bone == NULL || arma == NULL)
		return 0;
	if(bone == arma)
		return 1;
	if(bone->id->length <= arma->id->length)
		return 0;
	if(! BinPath_is_prefixe(arma->id, bone->id))
		return 0;
	if(! BinPath_get(bone->id, arma->id->length, &val))
		return 0;
	return val;
}

static int BinPath_calculate_aux(Bone cur, BinPath prefixe, int step, int val)
{
	unsigned char * path = NULL;
	if(cur == NULL)
		return 1;
	if(val >= 0)
	{
		if(step > 0)
		{
			if((path = (unsigned char *)calloc(((step / 8) + ((step % 8) > 0)), sizeof(unsigned char))) == NULL)
				return 0;
		}
		BinPath_clear(cur->id);
		cur->id->length = step;
		cur->id->path = path;
		if(step > 0)
		{
			if(! BinPath_affect_prefixe(cur->id, prefixe, -1))
				return 0;
			if(! BinPath_affect(cur->id, step - 1, val))
				return 0;
		}
	}
	if(! BinPath_calculate_aux(cur->child, cur->id, step + 1, 1))
		return 0;
	if(! BinPath_calculate_aux(cur->brother, cur->id, step + 1, 0))
		return 0;
	return 1;
	
}

int BinPath_calculate(Bone arma)
{
	return BinPath_calculate_aux(arma, NULL, 0, 0);
}

int BinPath_calculate_from(Bone arma)
{
	if(arma->id == NULL)
		return BinPath_calculate(arma);
	return BinPath_calculate_aux(arma, arma->id, arma->id->length, -1);
}

Bone BinPath_get_bone(BinPath id, Bone arma)
{
	int val;
	int i;
	for(i = 0; BinPath_get(id, i, &val); ++i)
	{
		if(arma == NULL)
			return NULL;
		if(val)
			arma = arma->child;
		else
			arma = arma->brother;
	}
	return arma;
}

Bone BinPath_get_next(BinPath id, Bone current)
{
	int val;
	int bit;
	if(current == NULL)
		return NULL;
	if(current->id == NULL)
		bit = 0;
	else
		bit = current->id->length;
	if(! (BinPath_is_prefixe(current->id, id)))
		return NULL;
	if(! BinPath_get(id, bit, &val))
		return NULL;
	if(val)
		return current->child;
	return current->brother;
}

Bone Bone_get_next(Bone target, Bone current)
{
	if(target == NULL)
		return NULL;
	return BinPath_get_next(target->id, current);
}

void Bone_aff_debug_id(Bone bone)
{
	if(bone == NULL)
		return;
	BinPath_aff_debug(bone->id);
	Bone_aff_debug_id(bone->brother);
	Bone_aff_debug_id(bone->child);
}

int Bone_cmp(const void * a, const void * b)
{
	Bone first = NULL;
	Bone second = NULL;
	int i;
	int f_val;
	int s_val;
	
	if((a == NULL) ^ (b == NULL))
	{
		if(a == NULL)
			return -1;
		return 1;
	}
	if(a == NULL)
		return 0;
	first = *((Bone *)a);
	second = *((Bone *)b);
	for(i = 0; BinPath_get(first->id, i, &f_val) && BinPath_get(second->id, i, &s_val); ++i)
	{
		if(f_val < s_val)
			return -1;
		else if(f_val > s_val)
			return 1;
	}
	if(first->id->length < second->id->length)
		return -1;
	else if(first->id->length > second->id->length)
		return 1;
	return 0;
}

/* BoneProp */

int ChainList_create_BoneProp(ChainList * list)
{
	return ChainList_create(list, &BoneProp_create_forlist, &BoneProp_free_forlist, &BoneProp_affect_forlist, &BoneProp_eq_forlist);
}

int BoneProp_create(BoneProp * prop, BoneProp_Type type, void * value)
{
	if(prop == NULL)
		return 0;
	switch(type)
	{
		case BONEPROP_MIRROR:
		prop->mirror_bone = (Bone)value;
		break;
		
		default:
		return 0;
	}
	prop->type = type;
	return 1;
}

int BoneProp_create_forlist(void * elem_adress, void * value)
{
	BoneProp * elem = NULL;
	if((elem = (BoneProp *)malloc(sizeof(BoneProp))) == NULL)
		return 0;
	*elem = *((BoneProp *)value);
	*((BoneProp **)elem_adress) = elem;
	return 1;
}

void BoneProp_free_forlist(void * elem_adress)
{
	BoneProp ** prop = (BoneProp **)elem_adress;
	if(prop == NULL || *prop == NULL)
		return;
	free(*prop);
	*prop = NULL;
}

int BoneProp_affect_forlist(void * dest_adress, void * src)
{
	if(dest_adress == NULL || src == NULL)
		return 0;
	*((BoneProp *)dest_adress) = *((BoneProp *)src);
	return 1;
}

int BoneProp_eq_forlist(void * in_list, void * elem)
{
	BoneProp * il = (BoneProp *)in_list;
	BoneProp * e = (BoneProp *)elem;
	if(e == il)
		return 1;
	if(e->type != il->type)
		return 0;
	switch(e->type)
	{
		case BONEPROP_MIRROR:
		return e->mirror_bone == il->mirror_bone;
		
		default:
		return 0;
	}
	return 1;
}

int BoneProp_eq_is_mirror(void * in_list, void * elem)
{
	BoneProp * il = (BoneProp *)in_list;
	if(il == NULL)
		return 0;
	return il->type == BONEPROP_MIRROR;
}

/* BoneProp use */

Bone Bone_get_mirror(Bone bone)
{
	BoneProp prop;
	if(bone == NULL)
		return NULL;
	if(! ChainList_get_with_function(bone->prop_list, &BoneProp_eq_is_mirror, NULL, &prop))
		return NULL;
	return prop.mirror_bone;
}

int Bone_make_mirror(Bone bone, Bone mirror)
{
	BoneProp prop;
	if(bone == NULL || mirror == NULL)
		return 0;
	if(Bone_get_mirror(bone) != NULL || Bone_get_mirror(mirror) != NULL)
		return 0;
	if(! BoneProp_create(&prop, BONEPROP_MIRROR, mirror)
	|| ! ChainList_append_head(bone->prop_list, &prop))
		return 0;
	if(! BoneProp_create(&prop, BONEPROP_MIRROR, bone)
	|| ! ChainList_append_head(mirror->prop_list, &prop))
		return 0;
	return 1;
}

int Bone_break_mirror(Bone bone)
{
	Bone mirror = NULL;
	if(bone == NULL)
		return 0;
	if((mirror = Bone_get_mirror(bone)) == NULL)
		return 0;
	return ChainList_delete_if_exists_with_function(bone->prop_list, &BoneProp_eq_is_mirror, NULL)
	&& ChainList_delete_if_exists_with_function(mirror->prop_list, &BoneProp_eq_is_mirror, NULL);
}

int Bone_add_mirror(Bone bone, Bone parent)
{
	Bone mirror = NULL;
	if(parent == NULL)
		return 0;
	if(! Bone_append(&parent, &mirror))
		return 0;
	if(! Bone_make_mirror(bone, mirror))
		return 0;
	mirror->start.x = -bone->start.x;
	mirror->start.y = bone->start.y;
	mirror->end.x = -bone->end.x;
	mirror->end.y = bone->end.y;
	return 1;
}

/* Fight functions */

static void Bone_make_init_aux(Bone bone)
{
	if(bone == NULL)
		return;
	bone->angle = bone->start_angle;
	Bone_make_init_aux(bone->child);
	Bone_make_init_aux(bone->brother);
}

void Bone_make_init(Bone bone)
{
	Bone_make_init_aux(bone);
	/*Bone_rotate_hitbox(bone);*/
}

double Bone_get_power(Bone bone)
{
	return 6;
}

int Bone_make_hitbox(Bone bone)
{
	if(bone == NULL)
		return 1;
	HitBox_free(&(bone->hitbox));
	if(! HitBox_generate(&(bone->hitbox), bone->start, bone->end, bone->shape))
		return 0;
	if(! Bone_make_hitbox(bone->child))
		return 0;
	if(! Bone_make_hitbox(bone->brother))
		return 0;
	return 1;
}

double Bone_get_vector_distance_carre(Bone bone)
{
	return distance_carre(bone->last_real_end.x, bone->last_real_end.y, bone->real_end.x, bone->real_end.y);
}

static void Bone_rotate_hitbox_aux(Bone bone, double angle)
{
	if(bone == NULL)
		return;
	HitBox_rotate(bone->hitbox, correct_degre_angle(bone->angle + angle));
	Bone_rotate_hitbox_aux(bone->child, correct_degre_angle(bone->angle + angle));
	Bone_rotate_hitbox_aux(bone->brother, angle);
}

void Bone_rotate_hitbox(Bone bone)
{
	Bone_rotate_hitbox_aux(bone, 0);
}

static int Bone_get_fight_dimensions_aux(Bone bone, int * x_min, int * y_min, int * x_max, int * y_max, int x, int y, double angle)
{
	SDL_Rect next, start, origin;
	Bone child = NULL;
	
	origin.x = origin.y = 0;
	if(bone == NULL)
		return 1;
	start = get_SDL_Rect_by_rotation(bone->start, origin, degre_to_rad(angle));
	x += (start.x - bone->start.x);
	y += (start.y - bone->start.y);
	angle = correct_degre_angle(angle + bone->angle);
	
	next = get_SDL_Rect_by_rotation(bone->end, bone->start, degre_to_rad(angle));
	HitBox_dimensions_aux(bone->hitbox, x_min, y_min, x_max, y_max, bone->start.x + x, bone->start.y + y);
	for(child = bone->child; child != NULL; child = child->brother)
	{
		if(! Bone_get_fight_dimensions_aux(child, x_min, y_min, x_max, y_max, x + next.x, y + next.y, angle))
			return 0;
	}
	
	return 1;
}

int Bone_get_fight_dimensions(Bone bone, int * x_min, int * y_min, int * x_max, int * y_max)
{
	if(bone == NULL)
		return 0;
	if(x_min != NULL)
		*x_min = bone->start.x;
	if(x_max != NULL)
		*x_max = bone->start.x;
	if(y_min != NULL)
		*y_min = bone->start.y;
	if(y_max != NULL)
		*y_max = bone->start.y;
	if(! Bone_get_fight_dimensions_aux(bone, x_min, y_min, x_max, y_max, 0, 0, 0))
		return 0;
	return 1;
}

static int HitBox_make_impacted_hitbox(HitBox box, SDL_Rect * box_start, HitBox hit, SDL_Rect * hit_start)
{
	HitBox child;
	if(box == NULL)
		return 0;
	if(circle_collision(box_start->x + box->rotated_center.x, box_start->y + box->rotated_center.y, box->radius, hit_start->x + hit->rotated_center.x, hit_start->y + hit->rotated_center.y, hit->radius))
		return 1;
	for(child = box->child; child != NULL; child = child->brother)
	{
		if(circle_collision_segment(hit_start->x + hit->rotated_center.x, hit_start->y + hit->rotated_center.y, hit->radius + (box->radius + child->radius) / 2,
		box_start->x + box->rotated_center.x, box_start->y + box->rotated_center.y,
		box_start->x + child->rotated_center.x, box_start->y + child->rotated_center.y))
			return 1;
		if(HitBox_make_impacted_hitbox(child, box_start, hit, hit_start))
			return 1;
	}
	return 0;
}

#include "draw.h"

static int Bone_make_impacted_hitbox(Bone bone, HitBox hit, SDL_Rect * real_start)
{
	if(hit == NULL)
		return 0;
	if(bone->hit_delay <= 0
	&& speed_circle_collision_box(	real_start->x + hit->rotated_center.x, real_start->y + hit->rotated_center.y, hit->radius,
					bone->shape_image->last_rotated_coord.x, bone->shape_image->last_rotated_coord.y,
					bone->shape_image->last_rotated_coord.x + bone->shape_image->last_rotated_coord.w,
					bone->shape_image->last_rotated_coord.y + bone->shape_image->last_rotated_coord.h))
	{
		if(HitBox_make_impacted_hitbox(bone->hitbox, &(bone->real_start), hit, real_start))
		{
			draw_hitbox(bone->hitbox, bone->real_start.x, bone->real_start.y);
			draw_hitbox(hit, real_start->x, real_start->y);
			bone->hit_delay = HIT_DELAY;
			Image_draw_impact(bone->shape_image);
			return 1;
		}
	}
	if(Bone_make_impacted_hitbox(bone, hit->child, real_start)
	|| Bone_make_impacted_hitbox(bone, hit->brother, real_start))
		return 1;
	return 0;
}

int Bone_make_impacted(Bone bone, Bone hit)
{
	int impact = 0;
	if(bone == NULL || hit == NULL)
		return 0;
	if(bone->shape_image != NULL && hit->shape_image != NULL
	&& box_collision(bone->shape_image->last_rotated_coord.x, bone->shape_image->last_rotated_coord.y,
			bone->shape_image->last_rotated_coord.x + bone->shape_image->last_rotated_coord.w,
			bone->shape_image->last_rotated_coord.y + bone->shape_image->last_rotated_coord.h,
			hit->shape_image->last_rotated_coord.x, hit->shape_image->last_rotated_coord.y,
			hit->shape_image->last_rotated_coord.x + hit->shape_image->last_rotated_coord.w,
			hit->shape_image->last_rotated_coord.y + hit->shape_image->last_rotated_coord.h))
	{
		if(Bone_make_impacted_hitbox(bone, hit->hitbox, &(hit->real_start)))
		{
			/*draw_bone_part_boneRGBA(get_screen(), bone->real_start.x, bone->real_start.y, bone->real_end.x, bone->real_end.y, 255, 0, 0, 255);
			draw_bone_part_boneRGBA(get_screen(), hit->real_start.x, hit->real_start.y, hit->real_end.x, hit->real_end.y, 0, 255, 0, 255);*/
			impact = 1;
		}
	}
	return impact | Bone_make_impacted(bone->child, hit) | Bone_make_impacted(bone->brother, hit);
}