#include "dialog.h"

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include "button.h"
#include "image.h"
#include "color.h"

Dialog dial_global;

/* Surface functions */

int TextSurface_create_va(SDL_Surface ** surface, TTF_Font * font, SDL_Color color, const char * format, va_list ap)
{
	int size = 0;
	char * chaine = NULL;
	va_list cp;
	va_copy(cp, ap);
	if((size = vsnprintf(chaine, size, format, cp)) < 0)
		return 0;
	++size;
	if((chaine = (char *)malloc(size * sizeof(char))) == NULL)
		return 0;
	if((size = vsnprintf(chaine, size, format, ap)) < 0)
	{
		free(chaine);
		return 0;
	}
	if((*surface = TTF_RenderUTF8_Blended(font, chaine, color)) == NULL)
	{
		free(chaine);
		return 0;
	}
	free(chaine);
	return 1;
}

void print_text(SDL_Surface * screen, SDL_Color color, const char * format, ...)
{
	va_list ap;
	SDL_Surface * text = NULL;
	SDL_Rect pos;
	
	va_start(ap, format);
	if(! TextSurface_create_va(&text, dial_global.font, color, format, ap))
		return;
	pos.x = screen->w / 2 - text->w / 2;
	pos.y = screen->h / 2 - text->h / 2;
	SDL_BlitSurface(text, NULL, screen, &pos);
	SDL_FreeSurface(text);
	va_end(ap);
}

/* Dialog functions */

int Dialog_create(Dialog * dial, const char * path, int size, SDL_Color color)
{
	TTF_Font * font = NULL;
	Input input = NULL;
	if(dial == NULL || path == NULL)
		return 0;
	if(! Input_create(&input))
		return 0;
	Input_set_select(input, INPUT_SELECT);
	if(! Input_set_font(input, path, size, color))
	{
		Input_free(&input);
		return 0;
	}
	if((font = TTF_OpenFont(path, size)) == NULL)
	{
		Input_free(&input);
		return 0;
	}
	dial->font = font;
	dial->color = color;
	dial->input = input;
	return 1;
}

void Dialog_free(Dialog * dial)
{
	if(dial == NULL)
		return;
	Input_free(&(dial->input));
	if(dial->font != NULL)
	{
		TTF_CloseFont(dial->font);
		dial->font = NULL;
	}
}

static int Input_define_on_screen(SDL_Surface * screen, int height, Input input)
{
	SDL_Surface * background = NULL;
	if(! SDL_GetRectangleRGB(&background, (screen->w * 3) / 4, height, 127, 127, 127))
		return 0;
	if(! Input_set_box(input, background))
	{
		SDL_FreeSurface(background);
		return 0;
	}
	SDL_FreeSurface(background);
	Input_set_origin(input, screen->w / 8, screen->h / 2);
	return 1;
}

int Dialog_print_va(SDL_Surface * screen, Dialog * dial, const char * format, char ** alloc_text, va_list ap)
{
	SDL_Surface * text = NULL;
	SDL_Rect text_pos;
	SDL_Event event;
	Buttons buttons;
	int buttons_choix = -1;
	int need_refresh = 1;
	int success = 1;
	int exit_input = 0;
	Image back_screen = NULL;
	
	if(screen == NULL || dial == NULL || format == NULL)
		return 0;
	if(! Image_create_from_surface_copy(&back_screen, screen))
		return 0;
	if(! Buttons_create(&buttons, screen->w, (alloc_text == NULL) ? (screen->h / 2) : ((screen->h * 2) / 3), 1, "Ok"))
	{
		Image_free(&back_screen);
		return 0;
	}
	if(! TextSurface_create_va(&text, dial->font, dial->color, format, ap))
	{
		Image_free(&back_screen);
		Buttons_free(&buttons);
		return 0;
	}
	Input_redefine(dial->input, "");
	Input_set_select(dial->input, INPUT_SELECT);
	if(alloc_text != NULL)
	{
		if(! Input_define_on_screen(screen, (text->h * 4) / 3, dial->input))
		{
			Image_free(&back_screen);
			return 0;
		}
	}
	text_pos.y = (screen->h / 2) - (text->h * 2);
	text_pos.x = (screen->w / 2) - (text->w / 2);
	
	while(buttons_choix == -1 && success && ! exit_input)
	{
		if(need_refresh)
		{
			Image_print_on_surface(screen, back_screen);
			SDL_BlitSurface(text, NULL, screen, &text_pos);
			if(alloc_text != NULL)
				Input_draw(dial->input, screen);
			Buttons_draw(&buttons, screen);
			SDL_Flip(screen);
			need_refresh = 0;
		}
		SDL_WaitEvent(&event);
		switch(event.type)
		{
			case SDL_QUIT:
			success = 0;
			break;
			
			case SDL_KEYDOWN:
			switch(event.key.keysym.sym)
			{
				case SDLK_ESCAPE:
				success = 0;
				break;
				
				case SDLK_RETURN:
				buttons_choix = 0;
				break;
				
				default:
				break;
			}
			break;
			
			default:
			break;
		}
		need_refresh |= Buttons_get_event(&buttons, &event, &buttons_choix);
		if(alloc_text != NULL)
			need_refresh |= Input_event(dial->input, &event, &exit_input);
	}
	if(alloc_text != NULL && success)
	{
		success &= Input_get_allocated_text(dial->input, alloc_text);
		Input_redefine(dial->input, "");
	}
	Image_print_on_surface(screen, back_screen);
	SDL_Flip(screen);
	Image_free(&back_screen);
	Buttons_free(&buttons);
	SDL_FreeSurface(text);
	
	return success;
}

int Dialog_print(SDL_Surface * screen, Dialog * dial, const char * format, ...)
{
	int retour;
	va_list ap;
	va_start(ap, format);
	retour = Dialog_print_va(screen, dial, format, NULL, ap);
	va_end(ap);
	return retour;
}

int Dialog_ask_text(SDL_Surface * screen, Dialog * dial, const char * format, char ** alloc_text, ...)
{
	int retour;
	va_list ap;
	va_start(ap, alloc_text);
	retour = Dialog_print_va(screen, dial, format, alloc_text, ap);
	va_end(ap);
	return retour;
}

/* Dialog global */

int Dialog_global_init(const char * path, int size, SDL_Color color)
{
	return Dialog_create(&dial_global, path, size, color);
}

void Dialog_global_free()
{
	Dialog_free(&dial_global);
}

int Dialog_global_print(SDL_Surface * screen, const char * format, ...)
{
	int retour;
	va_list ap;
	va_start(ap, format);
	retour = Dialog_print_va(screen, &dial_global, format, NULL, ap);
	va_end(ap);
	return retour;
}

int Dialog_global_ask_text(SDL_Surface * screen, const char * format, char ** alloc_text, ...)
{
	int retour;
	va_list ap;
	va_start(ap, alloc_text);
	retour = Dialog_print_va(screen, &dial_global, format, alloc_text, ap);
	va_end(ap);
	return retour;
}