#include "image.h"

#include <SDL/SDL_rotozoom.h>
#include <SDL/SDL_gfxPrimitives.h>
#include "math.h"
#include "color.h"

static void pixelscpy(void * dest, void * src, int size)
{
	char * res_cur = NULL;
	char * src_cur = NULL;
	int i;
	for(i = 0, res_cur = dest, src_cur = (char *)src; i < size * sizeof(Uint32); ++i, ++res_cur, ++src_cur)
		*res_cur = *src_cur;
}

int SDL_CopySurface(SDL_Surface ** dest, SDL_Surface * src)
{
	SDL_Surface * res = NULL;
	if(dest == NULL)
		return 0;
	if(src == NULL)
	{
		*dest = NULL;
		return 1;
	}
	if((res = SDL_CreateRGBSurface(src->flags, src->w, src->h, 32, src->format->Rmask, src->format->Gmask, src->format->Bmask, src->format->Amask)) == NULL)
		return 0;
	pixelscpy(res->pixels, src->pixels, src->w * src->h);
	
	*dest = res;
	
	return 1;
}

int in_surface_area(int x, int y, SDL_Rect origin, SDL_Surface * surface)
{
	if(surface == NULL)
		return 0;
	return in_area(x, y, origin.x, origin.y, origin.x + surface->w, origin.y + surface->h);
}

int in_area(int x, int y, int ox, int oy, int fx, int fy)
{
	return x >= ox && y >= oy && x < fx && y < fy;
}

int in_SDL_Rect(int x, int y, SDL_Rect rect)
{
	return in_area(x, y, rect.x, rect.y, rect.x + rect.w, rect.y + rect.h);
}

void draw_rectangle(SDL_Surface * surface, int x, int y, int w, int h, int r, int g, int b)
{
	SDL_Rect zone;
	zone.x = x;
	zone.y = y;
	zone.w = w;
	zone.h = h;
	SDL_FillRect(surface, &zone, SDL_MapRGB(surface->format, r, g, b));
}

int SDL_GetRectangleRGB(SDL_Surface ** surface, int width, int height, int r, int g, int b)
{
	SDL_Surface * res = NULL;
	if(surface == NULL)
		return 0;
	if((res = SDL_CreateRGBSurface(SDL_HWSURFACE, width, height, 32, 0, 0, 0, 0)) == NULL)
		return 0;
	SDL_FillRect(res, NULL, SDL_MapRGB(res->format, r, g, b));
	*surface = res;
	return 1;
}

int SDL_GetMecaRectangleRGB(SDL_Surface ** surface, int width, int height, int r, int g, int b)
{
	SDL_Surface * res = NULL;
	if(! SDL_GetRectangleRGB(&res, width, height, 63, 63, 63))
		return 0;
	draw_rectangle(res, 0, 0, width - 2, height - 2, 196, 196, 196);
	draw_rectangle(res, 2, 2, width - 4, height - 4, 127, 127, 127);
	draw_rectangle(res, 4, 4, width - 8, height - 8, 63, 63, 63);
	draw_rectangle(res, 6, 6, width - 10, height - 10, 196, 196, 196);
	draw_rectangle(res, 6, 6, width - 12, height - 12, r, g, b);
	*surface = res;
	return 1;
}

void SDL_SetPixelOnSurfaceIfNotColorKey(SDL_Surface * surface, int x, int y, int intensity, SDL_Color color, SDL_Color color_key, SDL_Color (*affect)(SDL_Color, SDL_Color, int))
{
	SDL_Color get_color;
	if(x < 0 || y < 0 || x >= surface->w || y >= surface->h)
		return;
	SDL_Color_get_from_surface(&get_color, surface, x, y);
	if(SDL_Color_equal(get_color, color_key))
		return;
	SDL_Color_set_on_surface((affect == NULL) ? color : ((*affect)(color, get_color, intensity)), surface, x, y);
}

/* Image creation */

int Image_create(Image * image)
{
	Image res = NULL;
	
	if((res = (Image)malloc(sizeof(struct _Image))) == NULL)
		return 0;
	res->surface = NULL;
	res->coord.x = res->coord.y = 0;
	res->last_rotated = NULL;
	res->last_angle = 0;
	res->is_angle = 0;
	res->last_rotated_coord.x = res->last_rotated_coord.y = 0;
	res->last_rotated_coord.w = res->last_rotated_coord.h = 0;
	
	*image = res;
	return 1;
}

int Image_create_from_surface_copy(Image * image, SDL_Surface * surface)
{
	Image res = NULL;
	if(! Image_create(&res))
		return 0;
	if(! SDL_CopySurface(&(res->surface), surface))
	{
		Image_free(&res);
		return 0;
	}
	*image = res;
	return 1;
}

void Image_free(Image * image)
{
	if(image == NULL || *image == NULL)
		return;
	Image_clear(*image);
	free(*image);
	*image = NULL;
}

int Image_copy(Image * copy, Image image)
{
	Image res = NULL;
	if(image == NULL)
	{
		*copy = NULL;
		return 1;
	}
	if(! Image_create_from_surface_copy(&res, image->surface))
		return 0;
	res->coord = image->coord;
	res->use_color_key = image->use_color_key;
	res->color_key = image->color_key;
	*copy = res;
	return 1;
}

/* Image edition */

int Image_replace_surface_by_copy(Image image, SDL_Surface * surface)
{
	SDL_Surface * copy = NULL;
	if(image == NULL)
		return 0;
	if(! SDL_CopySurface(&copy, surface))
		return 0;
	Image_clear(image);
	image->surface = copy;
	return 1;
}

int Image_set_coord(Image image, int x, int y)
{
	if(image == NULL)
		return 0;
	if(image->coord.x == x && image->coord.y == y)
		return 0;
	image->coord.x = x;
	image->coord.y = y;
	return 1;
}

int Image_clear(Image image)
{
	if(image == NULL)
		return 0;
	if(image->surface == NULL)
		return 0;
	SDL_FreeSurface(image->surface);
	image->surface = NULL;
	Image_clear_rotate(image);
	
	return 1;
}

int Image_set_color_key(Image image, SDL_Color color_key)
{
	if(image == NULL)
		return 0;
	image->color_key = color_key;
	image->use_color_key = 1;
	if(image->surface != NULL)
		SDL_SetColorKey(image->surface, SDL_SRCCOLORKEY, SDL_MapRGB(image->surface->format, color_key.r, color_key.g, color_key.b));
	return 1;
}

void Image_fillRect(Image image, SDL_Rect * coord, int r, int g, int b)
{
	if(! Image_exists(image))
		return;
	SDL_FillRect(image->surface, coord, SDL_MapRGB(image->surface->format, r, g, b));
}

int transition_linear(int time, int start_time, int end_time, int start_data, int end_data)
{
	if(start_time == end_time)
		return end_data;
	if(time <= start_time)
		return start_data;
	if(time >= end_time)
		return end_data;
	return start_data + (((end_data - start_data) * (time - start_time)) / (end_time - start_time));
}

void Image_fillHorizontalDegrade(Image image, int start, int end, int start_r, int start_g, int start_b, int end_r, int end_g, int end_b, int (*transition)(int time, int start_time, int end_time, int start_data, int end_data))
{
	SDL_Rect zone;
	if(! Image_exists(image))
		return;
	if(transition == NULL)
		transition = &transition_linear;
	zone.x = 0;
	zone.w = Image_get_width(image);
	zone.h = 1;
	for(zone.y = start; zone.y <= end; ++(zone.y))
	{
		SDL_FillRect(image->surface, &zone, SDL_MapRGB(image->surface->format,
		(*transition)(zone.y, start, end, start_r, end_r),
		(*transition)(zone.y, start, end, start_g, end_g),
		(*transition)(zone.y, start, end, start_b, end_b)));
	}
}

void Image_fillCircleDegrade(Image image, int x, int y, int rad, int r, int g, int b)
{
	int i;
	if(rad <= 0)
		return;
	if(! Image_exists(image))
		return;
	for(i = 0; i < 51; ++i)
		filledCircleRGBA(image->surface, x, y, (i * rad) / 51, r, g, b, 5);
}

static int make_dark_intensity(int c, int o, int i)
{
	return (o * ((i * c + (255 - i) * 255)/ 255)) / 255;
}

static SDL_Color darker(SDL_Color new_color, SDL_Color on_surface, int intensity)
{
	return SDL_Color_create(make_dark_intensity(new_color.r, on_surface.r, intensity),
				make_dark_intensity(new_color.g, on_surface.g, intensity),
				make_dark_intensity(new_color.b, on_surface.b, intensity));
}

void Image_draw_impact(Image image)
{
	int i, j;
	int x, y;
	int imp, dist;
	SDL_Color impact;
	if(image == NULL || image->surface == NULL)
		return;
	x = rand() % image->surface->w;
	y = rand() % image->surface->h;
	impact = SDL_Color_create(rand() % 63, 0, rand() % 63);
	imp = carre(IMPACT_SIZE);
	for(i = -IMPACT_SIZE; i < IMPACT_SIZE + 1; ++i)
	{
		for(j = -IMPACT_SIZE; j < IMPACT_SIZE + 1; ++j)
		{
			if((dist = distance_carre(0, 0, i, j)) > imp)
				continue;
			SDL_SetPixelOnSurfaceIfNotColorKey(image->surface, i + x, j + y, (96 * (imp - dist)) / imp, impact, image->color_key, &darker);
		}
	}
	Image_clear_rotate(image);
}

void Image_clear_rotate(Image image)
{
	if(image == NULL)
		return;
	if(image->last_rotated != NULL)
	{
		SDL_FreeSurface(image->last_rotated);
		image->last_rotated = NULL;
		image->last_angle = 0;
		image->is_angle = 0;
	}
}

/* Image print */

int Image_print_on_surface(SDL_Surface * dest, Image image)
{
	if(image == NULL || image->surface == NULL || dest == NULL)
		return 0;
	SDL_BlitSurface(image->surface, NULL, dest, &(image->coord));
	return 1;
}

int Image_print_on_surface_with_add(SDL_Surface * dest, Image image, int add_x, int add_y)
{
	SDL_Rect last_coord;
	int retour;
	if(image == NULL || dest == NULL)
		return 0;
	last_coord = image->coord;
	image->coord.x += add_x;
	image->coord.y += add_y;
	retour = Image_print_on_surface(dest, image);
	image->coord = last_coord;
	return retour;
}

int Image_print_rotated_on_surface(SDL_Surface * dest, Image image, int add_x, int add_y, double rad_angle, int axe_x, int axe_y)
{
	SDL_Surface * rotated = NULL;
	int new_x, new_y;
	if(image == NULL || dest == NULL || image->surface == NULL)
		return 0;
	if(image->is_angle && rad_same(rad_angle, image->last_angle))
		rotated = image->last_rotated;
	else if(! image->is_angle && rad_same(rad_angle, 0))
	{
		image->last_angle = 0;
		image->is_angle = 1;
		if(! SDL_CopySurface(&rotated, image->surface))
			return 0;
		if(image->use_color_key)
			SDL_SetColorKey(rotated, SDL_SRCCOLORKEY, SDL_MapRGB(rotated->format, image->color_key.r, image->color_key.g, image->color_key.b));
	}
	else
		rotated = rotozoomSurface(image->surface, rad_to_degre(-rad_angle), 1, 0);
	
	if(rotated != image->last_rotated)
	{
		if(image->last_rotated != NULL)
			SDL_FreeSurface(image->last_rotated);
		image->last_rotated = rotated;
		image->last_angle = rad_angle;
		image->is_angle = 1;
	}
	
	get_coord_by_rotation(&new_x, &new_y, image->coord.x + add_x + image->surface->w / 2, image->coord.y + add_y + image->surface->h / 2, axe_x, axe_y, rad_angle);
	new_x -= rotated->w / 2;
	new_y -= rotated->h / 2;
	image->last_rotated_coord.x = new_x;
	image->last_rotated_coord.y = new_y;
	image->last_rotated_coord.w = rotated->w;
	image->last_rotated_coord.h = rotated->h;
	SDL_BlitSurface(rotated, NULL, dest, &(image->last_rotated_coord));
	/*SDL_FreeSurface(rotated);*/
	return 1;
}

/* Image info */

int Image_exists(Image image)
{
	return image != NULL && image->surface != NULL;
}

int Image_get_width(Image image)
{
	if(! Image_exists(image))
		return 0;
	return image->surface->w;
}

int Image_get_height(Image image)
{
	if(! Image_exists(image))
		return 0;
	return image->surface->h;
}

SDL_Surface * Image_get_surface(Image image)
{
	if(! Image_exists(image))
		return NULL;
	return image->surface;
}