#include "mini_menu.h"

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>

#include "image.h"
#include "math.h"

int Mini_Menu_create(Mini_Menu * menu)
{
	Mini_Menu res = NULL;
	
	if(menu == NULL)
		return 0;
	if((res = (Mini_Menu)malloc(sizeof(struct _Mini_Menu))) == NULL)
		return 0;
	
	res->origin.x = res->origin.y = 0;
	res->button_background = res->button_background_hover = NULL;
	res->text_images = NULL;
	res->functions = NULL;
	res->options_count = 0;
	res->last_hover = -1;
	
	*menu = res;
	
	return 1;
}

void Mini_Menu_free(Mini_Menu * menu)
{
	int i;
	if(menu == NULL || *menu == NULL)
		return;
		
	for(i = 0; i < (*menu)->options_count; ++i)
	{
		SDL_FreeSurface(*((*menu)->text_images + i));
	}
	if((*menu)->button_background != NULL)
		SDL_FreeSurface((*menu)->button_background);
	if((*menu)->button_background_hover != NULL)
		SDL_FreeSurface((*menu)->button_background_hover);
	if((*menu)->text_images != NULL)
		free((*menu)->text_images);
	if((*menu)->functions != NULL)
		free((*menu)->functions);
	free(*menu);
	*menu = NULL;
	
}

int Mini_Menu_set_origin(Mini_Menu menu, int x, int y)
{
	if(menu == NULL)
		return 0;
	if(menu->origin.x == x && menu->origin.y == y)
		return 0;
	menu->origin.x = x;
	menu->origin.y = y;
	
	return 1;
}

int Mini_Menu_set_origin_on_screen(Mini_Menu menu, int x, int y, SDL_Surface * screen)
{
	if(menu == NULL)
		return 0;
	return Mini_Menu_set_origin(menu, max_int(2, 0, min_int(2, x, screen->w - Mini_Menu_width(menu))), max_int(2, 0, min_int(2, y, screen->h - Mini_Menu_height(menu))));
}

int Mini_Menu_set_button_background(Mini_Menu menu, SDL_Surface * background, SDL_Surface * background_hover)
{
	SDL_Surface * back = NULL;
	SDL_Surface * back_hover = NULL;
	
	if(menu == NULL || background == NULL)
		return 0;
	if(! SDL_CopySurface(&back, background))
		return 0;
	if(! SDL_CopySurface(&back_hover, background_hover))
	{
		if(back != NULL)
			SDL_FreeSurface(back);
		return 0;
	}
	
	if(menu->button_background != NULL)
	{
		SDL_FreeSurface(menu->button_background);
		menu->button_background = NULL;
	}
	if(menu->button_background_hover != NULL)
	{
		SDL_FreeSurface(menu->button_background_hover);
		menu->button_background_hover = NULL;
	}
	menu->button_background = back;
	menu->button_background_hover = back_hover;
	
	return 1;
}

int Mini_Menu_configure(Mini_Menu menu, const char * font_path, int font_size, SDL_Color color, int count, ...)
{
	va_list ap;
	SDL_Surface ** text_images = NULL;
	char ** functions = NULL;
	int i = 0;
	int success = 0;
	
	TTF_Font * font = NULL;
	if((font = TTF_OpenFont(font_path, font_size)) == NULL)
	{
		fprintf(stderr, "\"%s\" font not found\n", font_path);
		return 0;
	}
	va_start(ap, count);
	
	if((text_images = (SDL_Surface **)calloc(count, sizeof(SDL_Surface *))) == NULL)
		goto free_zone;
	if((functions = (char **)malloc(count * sizeof(void *))) == NULL)
		goto free_zone;
		
	for(i = 0; i < count; ++i)
	{
		if((*(text_images + i) = TTF_RenderUTF8_Blended(font, (char *)va_arg(ap, char *), color)) == NULL)
			goto free_zone;
		*(functions + i) = (char *)va_arg(ap, void *);
	}
	
	if(menu->text_images != NULL)
	{
		for(i = 0; i < menu->options_count; ++i)
		{
			if(*(menu->text_images + i) != NULL)
				SDL_FreeSurface(*(menu->text_images + i));
			*(menu->text_images + i) = NULL;
		}
		free(menu->text_images);
		menu->text_images = NULL;
		i = count;
	}
	if(menu->functions != NULL)
		free(menu->functions);
			
	menu->text_images = text_images;
	menu->functions = (void *)functions;
	menu->options_count = count;
	menu->last_hover = -1;
	success = 1;
	
	goto success_zone;
	
	free_zone:;
	
	va_end(ap);
	
	if(text_images != NULL)
	{
		for(--i; i >= 0; --i)
		{
			if(*(text_images + i) != NULL)
				SDL_FreeSurface(*(text_images + i));
		}
		free(text_images);
	}
	if(functions != NULL)
		free(functions);
	
	success_zone:;
	
	if(font != NULL)
		TTF_CloseFont(font);
	return success;
}

void Mini_Menu_draw(Mini_Menu menu, SDL_Surface * screen)
{
	int i;
	SDL_Rect pos;
	if(menu == NULL || screen == NULL || menu->button_background == NULL)
		return;
	pos.x = menu->origin.x;
	pos.y = menu->origin.y;
	for(i = 0; i < menu->options_count; ++i, pos.y += menu->button_background->h)
	{
		if(i == menu->last_hover && menu->button_background_hover != NULL)
			SDL_BlitSurface(menu->button_background_hover, NULL, screen, &pos);
		else
			SDL_BlitSurface(menu->button_background, NULL, screen, &pos);
		if(*(menu->text_images + i) != NULL)
		SDL_BlitSurface(*(menu->text_images + i), NULL, screen, &pos);
	}
}

int Mini_Menu_event(Mini_Menu menu, SDL_Event * event, void * get_function, int * exit_menu)
{
	int need_refresh = 0;
	int hover = -1;
	if(menu == NULL || menu->button_background == NULL || menu->button_background->h <= 0)
		return 0;
	switch(event->type)
	{
		case SDL_MOUSEMOTION:
		if(event->motion.x <= menu->origin.x
		|| event->motion.y <= menu->origin.y
		|| event->motion.x > menu->origin.x + menu->button_background->w
		|| event->motion.y > menu->origin.y + (menu->button_background->h * menu->options_count))
			hover = -1;
		else
			hover = (event->motion.y - menu->origin.y) / menu->button_background->h;
		if(hover != menu->last_hover)
		{
			menu->last_hover = hover;
			need_refresh = 1;
		}
		break;
		
		case SDL_MOUSEBUTTONUP:
		switch(event->button.button)
		{
			case SDL_BUTTON_LEFT:
			if(menu->last_hover >= 0 && menu->last_hover < menu->options_count)
			{
				if(get_function != NULL)
					*((char **)get_function) = *((char **)menu->functions + menu->last_hover);
			}
			if(exit_menu != NULL)
				*exit_menu = 1;
			need_refresh = 1;
			break;
		}
		break;
	}
	return need_refresh;
}

int Mini_Menu_gest(Mini_Menu menu, SDL_Surface * screen, void * get_function, Key_Gest * key)
{
	int need_refresh = 1;
	int exit_menu = 0;
	SDL_Event event;
	menu->last_hover = -1;
	
	while(! exit_menu)
	{
		if(need_refresh)
		{
			Mini_Menu_draw(menu, screen);
			SDL_Flip(screen);
			need_refresh = 0;
		}
		SDL_WaitEvent(&event);
		need_refresh = Mini_Menu_event(menu, &event, get_function, &exit_menu);
		switch(event.type)
		{
			case SDL_QUIT:
			return 0;
			
			case SDL_KEYDOWN:
			switch(event.key.keysym.sym)
			{
				case SDLK_ESCAPE:
				return 0;
				
				default:
				break;
			}
			break;
			
			case SDL_MOUSEBUTTONUP:
			switch(event.button.button)
			{
				case SDL_BUTTON_LEFT:
				break;
				
				default:
				exit_menu = 1;
				break;
			}
			break;
		}
		Key_Gest_event(key, &event);
	}
	return 1;
}

int Mini_Menu_width(Mini_Menu menu)
{
	if(menu == NULL || menu->button_background == NULL)
		return 0;
	return menu->button_background->w;
}

int Mini_Menu_height(Mini_Menu menu)
{
	if(menu == NULL || menu->button_background == NULL)
		return 0;
	return menu->button_background->h * menu->options_count;
}