#include "moving_vector.h"

#include <SDL/SDL_gfxPrimitives.h>
#include "math.h"
#include <stdio.h>
#include <stdlib.h>
#include "image.h"
#include "color.h"

static int abs_int(int a)
{
	return (a < 0) ? (-a) : a;
}

/* MovingVector */

int MovingVector_create(MovingVector * vector, MovingVectorCoord direction, int power)
{
	vector->direction = direction;
	vector->power = power;
	
	vector->time = 0;
	return 1;
}

int MovingVector_define_direction(MovingVector * vector, MovingVectorCoord direction)
{
	if(vector->direction == direction)
		return 0;
	vector->direction = direction;
	return 1;
}

int MovingVector_define_direction_by_coord(MovingVector * vector, int center_x, int center_y, int x, int y)
{
	return MovingVector_define_direction(vector,
					((y < center_y && 2 * abs_int(y - center_y) > abs_int(x - center_x)) * MVC_UP)
					| ((y > center_y && 2 * abs_int(y - center_y) > abs_int(x - center_x)) * MVC_DOWN)
					| ((x > center_x && 2 * abs_int(x - center_x) > abs_int(y - center_y)) * MVC_RIGHT)
					| ((x < center_x && 2 * abs_int(x - center_x) > abs_int(y - center_y)) * MVC_LEFT));
}

int MovingVector_graphic_define(MovingVector * vector, SDL_Surface * screen)
{
	Image back_screen = NULL;
	SDL_Event event;
	int success = 1;
	int need_refresh = 1;
	int cont = 1;
	int is_edit = 0;
	
	if(! Image_create_from_surface_copy(&back_screen, screen))
		return 0;
	
	while(cont && success)
	{
		if(need_refresh)
		{
			Image_print_on_surface(screen, back_screen);
			MovingVector_draw(vector, screen, (is_edit) ? SDL_Color_create(27, 152, 63) : SDL_Color_create(152, 63, 27));
			SDL_Flip(screen);
			need_refresh = 0;
		}
		SDL_WaitEvent(&event);
		switch(event.type)
		{
			case SDL_QUIT:
			success = 0;
			break;
			
			case SDL_KEYDOWN:
			switch(event.key.keysym.sym)
			{
				case SDLK_ESCAPE:
				success = 0;
				break;
				
				default:
				break;
			}
			break;
			
			case SDL_MOUSEBUTTONDOWN:
			if(is_edit)
				break;
			is_edit = 1;
			need_refresh = 1;
			break;
			
			case SDL_MOUSEMOTION:
			if(! is_edit)
				break;
			need_refresh |= MovingVector_define_direction_by_coord(vector, screen->w / 2, screen->h / 2, event.motion.x, event.motion.y);
			break;
			
			case SDL_MOUSEBUTTONUP:
			if(is_edit)
				cont = 0;
			break;
			
			default:
			break;
		}
	}
	
	Image_print_on_surface(screen, back_screen);
	SDL_Flip(screen);
	Image_free(&back_screen);
	
	return success;
}

int MovingVector_copy(MovingVector * dest, MovingVector * src)
{
	if(dest == NULL || src == NULL)
		return 0;
	*dest = *src;
	dest->time = 0;
	return 1;
}

void MovingVector_draw(MovingVector * vector, SDL_Surface * screen, SDL_Color color)
{
	int mid_x, mid_y;
	int tx, ty;
	Sint16 x[4], y[4];
	mid_x = screen->w / 2;
	mid_y = screen->h / 2;
	if(! vector->direction)
		return;
	if(((vector->direction & MVC_UP) == MVC_UP)
	^ ((vector->direction & MVC_DOWN) == MVC_DOWN)
	^ ((vector->direction & MVC_RIGHT) == MVC_RIGHT)
	^ ((vector->direction & MVC_LEFT) == MVC_LEFT))
	{
		SDL_Rect zone;
		if(vector->direction & (MVC_UP | MVC_DOWN))
		{
			zone.x = mid_x - mid_x / 16;
			zone.y = mid_y - mid_y / 2;
			zone.w = mid_x / 8;
			zone.h = mid_y;
		}
		else
		{
			zone.x = mid_x - mid_x / 2;
			zone.y = mid_y - mid_y / 16;
			zone.w = mid_x;
			zone.h = mid_y / 8;
		}
		if(vector->direction & MVC_UP)
		{
			x[0] = mid_x - mid_x / 8;
			x[1] = mid_x;
			x[2] = mid_x + mid_x / 8;
			
			y[0] = mid_y - mid_y / 2;
			y[1] = mid_y - mid_y / 2 - mid_y / 8;
			y[2] = mid_y - mid_y / 2;
		}
		else if(vector->direction & MVC_DOWN)
		{
			x[0] = mid_x - mid_x / 8;
			x[1] = mid_x;
			x[2] = mid_x + mid_x / 8;
			
			y[0] = mid_y + mid_y / 2;
			y[1] = mid_y + mid_y / 2 + mid_y / 8;
			y[2] = mid_y + mid_y / 2;
		}
		else if(vector->direction & MVC_RIGHT)
		{			
			x[0] = mid_x + mid_x / 2;
			x[1] = mid_x + mid_x / 2 + mid_x / 8;
			x[2] = mid_x + mid_x / 2;
			
			y[0] = mid_y - mid_y / 8;
			y[1] = mid_y;
			y[2] = mid_y + mid_y / 8;
		}
		else
		{			
			x[0] = mid_x - mid_x / 2;
			x[1] = mid_x - mid_x / 2 - mid_x / 8;
			x[2] = mid_x - mid_x / 2;
			
			y[0] = mid_y - mid_y / 8;
			y[1] = mid_y;
			y[2] = mid_y + mid_y / 8;
		}
		SDL_FillRect(screen, &zone, SDL_MapRGB(screen->format, color.r, color.g, color.b));
		filledPolygonRGBA(screen, x, y, 3, color.r, color.g, color.b, 255);
	}
	else
	{
		double angle;
		if(vector->direction & MVC_UP)
		{
			if(vector->direction & MVC_RIGHT)
				angle = degre_to_rad(315);
			else
				angle = degre_to_rad(225);
		}
		else
		{
			if(vector->direction & MVC_RIGHT)
				angle = degre_to_rad(45);
			else
				angle = degre_to_rad(135);
		}
		get_coord_by_rotation(&tx, &ty, mid_x - mid_x / 2, mid_y - mid_y / 16, mid_x, mid_y, angle);
		x[0] = tx; y[0] = ty;
		get_coord_by_rotation(&tx, &ty, mid_x + mid_x / 2, mid_y - mid_y / 16, mid_x, mid_y, angle);
		x[1] = tx; y[1] = ty;
		get_coord_by_rotation(&tx, &ty, mid_x + mid_x / 2, mid_y + mid_y / 16, mid_x, mid_y, angle);
		x[2] = tx; y[2] = ty;
		get_coord_by_rotation(&tx, &ty, mid_x - mid_x / 2, mid_y + mid_y / 16, mid_x, mid_y, angle);
		x[3] = tx; y[3] = ty;
		filledPolygonRGBA(screen, x, y, 4, color.r, color.g, color.b, 255);
		
		get_coord_by_rotation(&tx, &ty, mid_x + mid_x / 2, mid_y - mid_y / 8, mid_x, mid_y, angle);
		x[0] = tx; y[0] = ty;
		get_coord_by_rotation(&tx, &ty, mid_x + mid_x / 2 + mid_x / 8, mid_y, mid_x, mid_y, angle);
		x[1] = tx; y[1] = ty;
		get_coord_by_rotation(&tx, &ty, mid_x + mid_x / 2, mid_y + mid_y / 8, mid_x, mid_y, angle);
		x[2] = tx; y[2] = ty;
		filledPolygonRGBA(screen, x, y, 3, color.r, color.g, color.b, 255);
	}
}

int MovingVector_get_next_move_add(MovingVector * vector, int * x, int * y)
{
	if(vector == NULL)
		return 0;
	if(vector->direction == MVC_NULL)
		return 0;
	++(vector->time);
	if(! (vector->direction ^ MVC_RIGHT))
	{
		if(x != NULL)
			*x = vector->power;
		if(y != NULL)
			*y = 0;
		return 1;
	}
	else if(! (vector->direction ^ MVC_LEFT))
	{
		if(x != NULL)
			*x = -vector->power;
		if(y != NULL)
			*y = 0;
		return 1;
	}
	/*else if(! (vector->direction ^ MVC_DOWN))
	{
		if(x != NULL)
			*x = 0;
		if(y != NULL)
			*y = vector->power;
		return 1;
	}*/
	else if(vector->direction & MVC_UP)
	{
		if(x != NULL)
		{
			if(vector->direction & MVC_RIGHT)
				*x = vector->power;
			else if(vector->direction & MVC_LEFT)
				*x = -vector->power;
		}
		if(y != NULL)
			*y = -5 * vector->power + vector->time;
		return 1;
	}
	else if(vector->direction & MVC_DOWN)
	{
		int add = 5 * vector->power - vector->time;
		if(x != NULL)
		{
			if(add < 0)
				add = 0;
			if(vector->direction & MVC_RIGHT)
				*x = add;
			else if(vector->direction & MVC_LEFT)
				*x = -add;
		}
		if(y != NULL)
			*y = 0;
		return add > 0;
	}
	return 0;
}

int MovingVector_start_move(MovingVector * vector)
{
	if(vector == NULL)
		return 0;
	vector->time = 0;
	return 1;
}

int MovingVector_continue_after_anim(MovingVector * vector)
{
	if(vector->direction & MVC_UP || vector->direction & MVC_DOWN)
		return 1;
	return 0;
}

MovingVectorType MovingVector_type(MovingVector * vector)
{
	if(vector == NULL || vector->direction == MVC_NULL)
		return MVT_NULL;
	if(MovingVector_continue_after_anim(vector))
		return MVT_CONTINUOUS;
	return MVT_STATIC;
}

int MovingVector_save(FILE * flow, MovingVector * vector)
{
	int version = 1;
	if(flow == NULL || vector == NULL)
		return 0;
	if(! fwrite(&version, sizeof(int), 1, flow))
		return 0;
	if(! fwrite(&(vector->direction), sizeof(MovingVectorCoord), 1, flow)
	|| ! fwrite(&(vector->power), sizeof(int), 1, flow))
		return 0;
	return 1;
}

int MovingVector_load(FILE * flow, MovingVector * vector)
{
	int version = 0;
	if(flow == NULL || vector == NULL)
		return 0;
	if(! fread(&version, sizeof(int), 1, flow))
		return 0;
	if(version != 1)
		return 0;
	if(! fread(&(vector->direction), sizeof(MovingVectorCoord), 1, flow)
	|| ! fread(&(vector->power), sizeof(int), 1, flow))
		return 0;
	return 1;
}

int MovingVector_combinable(MovingVector * first, MovingVector * second)
{
	unsigned int c;
	if(first == NULL || second == NULL)
		return 0;
	if((first->direction == MVC_NULL) || (second->direction == MVC_NULL))
		return 1;
	if(first->direction & second->direction)
		return 0;
	c = first->direction | second->direction;
	if(!(((c & MVC_DOWN) == MVC_DOWN) ^ ((c & MVC_UP) == MVC_UP)))
		return 0;
	if(!(((c & MVC_RIGHT) == MVC_RIGHT) ^ ((c & MVC_LEFT) == MVC_LEFT)))
		return 0;
	return 1;
}

int MovingVector_combine(MovingVector * res, MovingVector * first, MovingVector * second)
{
	if(res == NULL || first == NULL || second == NULL)
		return 0;
	if(! MovingVector_combinable(first, second))
		return 0;
	res->direction = first->direction | second->direction;
	res->power = (first->power > second->power) ? first->power : second->power;
	return 1;
}

int MovingVector_delete_direction(MovingVector * vector, MovingVectorCoord direction)
{
	if(vector == NULL)
		return 0;
	vector->direction &= ~direction;
	return 1;
}

int MovingVector_delete_continue_after_anim(MovingVector * vector)
{
	return MovingVector_delete_direction(vector, MVC_UP | MVC_DOWN);
}

int MovingVector_has_direction(MovingVector * vector, MovingVectorCoord direction)
{
	if(vector == NULL)
		return 0;
	return (vector->direction & direction) == direction;
}

void MovingVector_reverse(MovingVector * vector)
{
	if(vector == NULL)
		return;
	if(MovingVector_has_direction(vector, MVC_RIGHT))
	{
		MovingVector_delete_direction(vector, MVC_RIGHT);
		vector->direction |= MVC_LEFT;
	}
	else if(MovingVector_has_direction(vector, MVC_LEFT))
	{
		MovingVector_delete_direction(vector, MVC_LEFT);
		vector->direction |= MVC_RIGHT;
	}
}

/* MovingVectorKey */

int MovingVectorKey_create(MovingVectorKey * key)
{
	key->key = SDLK_ESCAPE;
	key->press_key = 0;
	if(! MovingVector_create(&(key->vector), MVC_NULL, 0))
		return 0;
	return 1;
}

int MovingVectorKey_add_press(MovingVectorKey * mkey, SDLKey key, MovingVector * vector)
{
	mkey->key = key;
	mkey->press_key = 1;
	if(! MovingVector_copy(&(mkey->vector), vector))
		return 0;
	return 1;
}

int MovingVectorKey_del_press(MovingVectorKey * key)
{
	if(! key->press_key)
		return 0;
	key->press_key = 0;
	return 1;
}

/* MovingVectorGest */

int MovingVectorGest_create(MovingVectorGest * gest, int power)
{
	if(! MovingVector_create(&(gest->vector), MVC_NULL, power)
	|| ! MovingVectorKey_create(&(gest->static_vector))
	|| ! MovingVectorKey_create(&(gest->continuous_vector)))
		return 0;
	return 1;
}

MovingVectorType MovingVectorGest_add_press_key(MovingVectorGest * gest, SDLKey key, MovingVector * vector)
{
	MovingVectorType type;
	if((type = MovingVector_type(vector)) == MVT_NULL)
		return MVT_NULL;
	switch(type)
	{
		case MVT_STATIC:
		MovingVector_delete_direction(&(gest->vector), gest->static_vector.vector.direction);
		MovingVectorKey_add_press(&(gest->static_vector), key, vector);
		MovingVector_combine(&(gest->vector), &(gest->vector), vector);
		if(gest->continuous_vector.press_key)
			return MVT_CONTINUOUS;
		return MVT_STATIC;
		
		case MVT_CONTINUOUS:
		if(gest->continuous_vector.press_key)
			return MVT_NULL;
		if(MovingVector_type(&(gest->vector)) == MVT_STATIC && ! MovingVector_combinable(&(gest->vector), vector))
		{
			MovingVector_delete_direction(&(gest->vector), gest->static_vector.vector.direction);
			gest->static_vector.press_key = 0;
		}
		if(! MovingVector_combine(&(gest->vector), &(gest->vector), vector))
			return MVT_NULL;
		MovingVectorKey_add_press(&(gest->continuous_vector), key, vector);
		gest->vector.time = 0;
		return MVT_CONTINUOUS;
		
		default:
		return MVT_NULL;
	}
	return MVT_NULL;
}

MovingVectorType MovingVectorGest_del_press_key(MovingVectorGest * gest, SDLKey key)
{
	MovingVectorType type;
	if((type = MovingVector_type(&(gest->vector))) == MVT_NULL)
		return MVT_NULL;
	switch(type)
	{
		case MVT_CONTINUOUS:
		if(gest->continuous_vector.press_key && gest->continuous_vector.key == key)
			return MovingVector_type(&(gest->vector));
		case MVT_STATIC:
		if(gest->static_vector.press_key && gest->static_vector.key == key)
		{
			MovingVector_delete_direction(&(gest->vector), gest->static_vector.vector.direction);
			gest->static_vector.press_key = 0;
			return MovingVector_type(&(gest->vector));
		}
		break;
		
		default:
		return MovingVector_type(&(gest->vector));
	}
	return MovingVector_type(&(gest->vector));
}

MovingVectorType MovingVectorGest_del_continuous(MovingVectorGest * gest)
{
	if(MovingVector_type(&(gest->vector)) != MVT_CONTINUOUS)
		return MovingVector_type(&(gest->vector));
	MovingVector_delete_direction(&(gest->vector), gest->continuous_vector.vector.direction);
	gest->continuous_vector.press_key = 0;
	gest->vector.time = 0;
	return MovingVector_type(&(gest->vector));
}